FFmpeg
aacenc.c
Go to the documentation of this file.
1 /*
2  * AAC encoder
3  * Copyright (C) 2008 Konstantin Shishkov
4  *
5  * This file is part of FFmpeg.
6  *
7  * FFmpeg is free software; you can redistribute it and/or
8  * modify it under the terms of the GNU Lesser General Public
9  * License as published by the Free Software Foundation; either
10  * version 2.1 of the License, or (at your option) any later version.
11  *
12  * FFmpeg is distributed in the hope that it will be useful,
13  * but WITHOUT ANY WARRANTY; without even the implied warranty of
14  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
15  * Lesser General Public License for more details.
16  *
17  * You should have received a copy of the GNU Lesser General Public
18  * License along with FFmpeg; if not, write to the Free Software
19  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
20  */
21 
22 /**
23  * @file
24  * AAC encoder
25  */
26 
27 /***********************************
28  * TODOs:
29  * add sane pulse detection
30  ***********************************/
31 #include <float.h>
32 
34 #include "libavutil/libm.h"
35 #include "libavutil/float_dsp.h"
36 #include "libavutil/mem.h"
37 #include "libavutil/opt.h"
38 #include "avcodec.h"
39 #include "codec_internal.h"
40 #include "encode.h"
41 #include "put_bits.h"
42 #include "mpeg4audio.h"
43 #include "sinewin.h"
44 #include "profiles.h"
45 #include "version.h"
46 
47 #include "aac.h"
48 #include "aactab.h"
49 #include "aacenc.h"
50 #include "aacenctab.h"
51 #include "aacenc_utils.h"
52 
53 #include "psymodel.h"
54 
55 /**
56  * List of PCE (Program Configuration Element) for the channel layouts listed
57  * in channel_layout.h
58  *
59  * For those wishing in the future to add other layouts:
60  *
61  * - num_ele: number of elements in each group of front, side, back, lfe channels
62  * (an element is of type SCE (single channel), CPE (channel pair) for
63  * the first 3 groups; and is LFE for LFE group).
64  *
65  * - pairing: 0 for an SCE element or 1 for a CPE; does not apply to LFE group
66  *
67  * - index: there are three independent indices for SCE, CPE and LFE;
68  * they are incremented irrespective of the group to which the element belongs;
69  * they are not reset when going from one group to another
70  *
71  * Example: for 7.0 channel layout,
72  * .pairing = { { 1, 0 }, { 1 }, { 1 }, }, (3 CPE and 1 SCE in front group)
73  * .index = { { 0, 0 }, { 1 }, { 2 }, },
74  * (index is 0 for the single SCE but goes from 0 to 2 for the CPEs)
75  *
76  * The index order impacts the channel ordering. But is otherwise arbitrary
77  * (the sequence could have been 2, 0, 1 instead of 0, 1, 2).
78  *
79  * Spec allows for discontinuous indices, e.g. if one has a total of two SCE,
80  * SCE.0 SCE.15 is OK per spec; BUT it won't be decoded by our AAC decoder
81  * which at this time requires that indices fully cover some range starting
82  * from 0 (SCE.1 SCE.0 is OK but not SCE.0 SCE.15).
83  *
84  * - config_map: total number of elements and their types. Beware, the way the
85  * types are ordered impacts the final channel ordering.
86  *
87  * - reorder_map: reorders the channels.
88  *
89  */
90 static const AACPCEInfo aac_pce_configs[] = {
91  {
93  .num_ele = { 1, 0, 0, 0 },
94  .pairing = { { 0 }, },
95  .index = { { 0 }, },
96  .config_map = { 1, TYPE_SCE, },
97  .reorder_map = { 0 },
98  },
99  {
100  .layout = AV_CHANNEL_LAYOUT_STEREO,
101  .num_ele = { 1, 0, 0, 0 },
102  .pairing = { { 1 }, },
103  .index = { { 0 }, },
104  .config_map = { 1, TYPE_CPE, },
105  .reorder_map = { 0, 1 },
106  },
107  {
108  .layout = AV_CHANNEL_LAYOUT_2POINT1,
109  .num_ele = { 1, 0, 0, 1 },
110  .pairing = { { 1 }, },
111  .index = { { 0 },{ 0 },{ 0 },{ 0 } },
112  .config_map = { 2, TYPE_CPE, TYPE_LFE },
113  .reorder_map = { 0, 1, 2 },
114  },
115  {
116  .layout = AV_CHANNEL_LAYOUT_2_1,
117  .num_ele = { 1, 0, 1, 0 },
118  .pairing = { { 1 },{ 0 },{ 0 } },
119  .index = { { 0 },{ 0 },{ 0 }, },
120  .config_map = { 2, TYPE_CPE, TYPE_SCE },
121  .reorder_map = { 0, 1, 2 },
122  },
123  {
124  .layout = AV_CHANNEL_LAYOUT_SURROUND,
125  .num_ele = { 2, 0, 0, 0 },
126  .pairing = { { 0, 1 }, },
127  .index = { { 0, 0 }, },
128  .config_map = { 2, TYPE_SCE, TYPE_CPE },
129  .reorder_map = { 2, 0, 1 },
130  },
131  {
132  .layout = AV_CHANNEL_LAYOUT_3POINT1,
133  .num_ele = { 2, 0, 0, 1 },
134  .pairing = { { 0, 1 }, },
135  .index = { { 0, 0 }, { 0 }, { 0 }, { 0 }, },
136  .config_map = { 3, TYPE_SCE, TYPE_CPE, TYPE_LFE },
137  .reorder_map = { 2, 0, 1, 3 },
138  },
139  {
140  .layout = AV_CHANNEL_LAYOUT_4POINT0,
141  .num_ele = { 2, 0, 1, 0 },
142  .pairing = { { 0, 1 }, { 0 }, { 0 }, },
143  .index = { { 0, 0 }, { 0 }, { 1 } },
144  .config_map = { 3, TYPE_SCE, TYPE_CPE, TYPE_SCE },
145  .reorder_map = { 2, 0, 1, 3 },
146  },
147  {
148  .layout = AV_CHANNEL_LAYOUT_4POINT1,
149  .num_ele = { 2, 0, 1, 1 },
150  .pairing = { { 0, 1 }, { 0 }, { 0 }, },
151  .index = { { 0, 0 }, { 0 }, { 1 }, { 0 } },
152  .config_map = { 4, TYPE_SCE, TYPE_CPE, TYPE_SCE, TYPE_LFE },
153  .reorder_map = { 2, 0, 1, 4, 3 },
154  },
155  {
156  .layout = AV_CHANNEL_LAYOUT_2_2,
157  .num_ele = { 1, 0, 1, 0 },
158  .pairing = { { 1 }, { 0 }, { 1 }, },
159  .index = { { 0 }, { 0 }, { 1 } },
160  .config_map = { 2, TYPE_CPE, TYPE_CPE },
161  .reorder_map = { 0, 1, 2, 3 },
162  },
163  {
164  .layout = AV_CHANNEL_LAYOUT_QUAD,
165  .num_ele = { 1, 0, 1, 0 },
166  .pairing = { { 1 }, { 0 }, { 1 }, },
167  .index = { { 0 }, { 0 }, { 1 } },
168  .config_map = { 2, TYPE_CPE, TYPE_CPE },
169  .reorder_map = { 0, 1, 2, 3 },
170  },
171  {
172  .layout = AV_CHANNEL_LAYOUT_5POINT0,
173  .num_ele = { 2, 0, 1, 0 },
174  .pairing = { { 0, 1 }, { 0 }, { 1 } },
175  .index = { { 0, 0 }, { 0 }, { 1 } },
176  .config_map = { 3, TYPE_SCE, TYPE_CPE, TYPE_CPE },
177  .reorder_map = { 2, 0, 1, 3, 4 },
178  },
179  {
180  .layout = AV_CHANNEL_LAYOUT_5POINT1,
181  .num_ele = { 2, 0, 1, 1 },
182  .pairing = { { 0, 1 }, { 0 }, { 1 }, },
183  .index = { { 0, 0 }, { 0 }, { 1 }, { 0 } },
184  .config_map = { 4, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_LFE },
185  .reorder_map = { 2, 0, 1, 4, 5, 3 },
186  },
187  {
189  .num_ele = { 2, 0, 1, 0 },
190  .pairing = { { 0, 1 }, { 0 }, { 1 } },
191  .index = { { 0, 0 }, { 0 }, { 1 } },
192  .config_map = { 3, TYPE_SCE, TYPE_CPE, TYPE_CPE },
193  .reorder_map = { 2, 0, 1, 3, 4 },
194  },
195  {
197  .num_ele = { 2, 0, 1, 1 },
198  .pairing = { { 0, 1 }, { 0 }, { 1 }, },
199  .index = { { 0, 0 }, { 0 }, { 1 }, { 0 } },
200  .config_map = { 4, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_LFE },
201  .reorder_map = { 2, 0, 1, 4, 5, 3 },
202  },
203  {
204  .layout = AV_CHANNEL_LAYOUT_6POINT0,
205  .num_ele = { 2, 0, 2, 0 },
206  .pairing = { { 0, 1 }, { 0 }, { 1, 0 } },
207  .index = { { 0, 0 }, { 0 }, { 1, 1 } },
208  .config_map = { 4, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_SCE },
209  .reorder_map = { 2, 0, 1, 4, 5, 3 },
210  },
211  {
213  .num_ele = { 2, 0, 1, 0 },
214  .pairing = { { 1, 1 }, { 0 }, { 1 } },
215  .index = { { 0, 1 }, { 0 }, { 2 }, },
216  .config_map = { 3, TYPE_CPE, TYPE_CPE, TYPE_CPE, },
217  .reorder_map = { 2, 3, 0, 1, 4, 5 },
218  },
219  {
220  .layout = AV_CHANNEL_LAYOUT_HEXAGONAL,
221  .num_ele = { 2, 0, 2, 0 },
222  .pairing = { { 0, 1 }, { 0 }, { 1, 0 } },
223  .index = { { 0, 0 }, { 0 }, { 1, 1 } },
224  .config_map = { 4, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_SCE },
225  .reorder_map = { 2, 0, 1, 3, 4, 5 },
226  },
227  {
228  .layout = AV_CHANNEL_LAYOUT_6POINT1,
229  .num_ele = { 2, 0, 2, 1 },
230  .pairing = { { 0, 1 }, { 0 }, { 1, 0 }, },
231  .index = { { 0, 0 }, { 0 }, { 1, 1 }, { 0 } },
232  .config_map = { 5, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_SCE, TYPE_LFE },
233  .reorder_map = { 2, 0, 1, 5, 6, 4, 3 },
234  },
235  {
237  .num_ele = { 2, 0, 2, 1 },
238  .pairing = { { 0, 1 },{ 0 },{ 1, 0 }, },
239  .index = { { 0, 0 },{ 0 },{ 1, 1 },{ 0 } },
240  .config_map = { 5, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_SCE, TYPE_LFE },
241  .reorder_map = { 2, 0, 1, 4, 5, 6, 3 },
242  },
243  {
245  .num_ele = { 2, 0, 1, 1 },
246  .pairing = { { 1, 1 }, { 0 }, { 1 }, },
247  .index = { { 0, 1 }, { 0 }, { 2 }, { 0 }, },
248  .config_map = { 4, TYPE_CPE, TYPE_CPE, TYPE_CPE, TYPE_LFE, },
249  .reorder_map = { 3, 4, 0, 1, 5, 6, 2 },
250  },
251  {
252  .layout = AV_CHANNEL_LAYOUT_7POINT0,
253  .num_ele = { 2, 0, 2, 0 },
254  .pairing = { { 0, 1 }, { 0 }, { 1, 1 }, },
255  .index = { { 0, 0 }, { 0 }, { 2, 1 }, },
256  .config_map = { 4, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_CPE },
257  .reorder_map = { 2, 0, 1, 3, 4, 5, 6 },
258  },
259  {
261  .num_ele = { 3, 0, 1, 0 },
262  .pairing = { { 0, 1, 1 }, { 0 }, { 1 }, },
263  .index = { { 0, 0, 1 }, { 0 }, { 2 }, },
264  .config_map = { 4, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_CPE },
265  .reorder_map = { 2, 3, 4, 0, 1, 5, 6 },
266  },
267  {
268  .layout = AV_CHANNEL_LAYOUT_7POINT1,
269  .num_ele = { 2, 0, 2, 1 },
270  .pairing = { { 0, 1 }, { 0 }, { 1, 1 }, },
271  .index = { { 0, 0 }, { 0 }, { 2, 1 }, { 0 } },
272  .config_map = { 5, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_CPE, TYPE_LFE },
273  .reorder_map = { 2, 0, 1, 4, 5, 6, 7, 3 },
274  },
275  {
277  .num_ele = { 3, 0, 1, 1 },
278  .pairing = { { 0, 1, 1 }, { 0 }, { 1 }, },
279  .index = { { 0, 0, 1 }, { 0 }, { 2 }, { 0 }, },
280  .config_map = { 5, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_CPE, TYPE_LFE },
281  .reorder_map = { 2, 4, 5, 0, 1, 6, 7, 3 },
282  },
283  {
285  .num_ele = { 3, 0, 1, 1 },
286  .pairing = { { 0, 1, 1 }, { 0 }, { 1 } },
287  .index = { { 0, 0, 1 }, { 0 }, { 2 }, { 0 } },
288  .config_map = { 5, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_CPE, TYPE_LFE },
289  .reorder_map = { 2, 6, 7, 0, 1, 4, 5, 3 },
290  },
291  {
292  .layout = AV_CHANNEL_LAYOUT_OCTAGONAL,
293  .num_ele = { 2, 0, 3, 0 },
294  .pairing = { { 0, 1 }, { 0 }, { 1, 1, 0 }, },
295  .index = { { 0, 0 }, { 0 }, { 1, 2, 1 }, },
296  .config_map = { 5, TYPE_SCE, TYPE_CPE, TYPE_CPE, TYPE_CPE, TYPE_SCE },
297  .reorder_map = { 2, 0, 1, 6, 7, 3, 4, 5 },
298  },
299 };
300 
301 static void put_pce(PutBitContext *pb, AVCodecContext *avctx)
302 {
303  int i, j;
304  AACEncContext *s = avctx->priv_data;
305  AACPCEInfo *pce = &s->pce;
306  const int bitexact = avctx->flags & AV_CODEC_FLAG_BITEXACT;
307  const char *aux_data = bitexact ? "Lavc" : LIBAVCODEC_IDENT;
308 
309  put_bits(pb, 4, 0);
310 
311  put_bits(pb, 2, avctx->profile);
312  put_bits(pb, 4, s->samplerate_index);
313 
314  put_bits(pb, 4, pce->num_ele[0]); /* Front */
315  put_bits(pb, 4, pce->num_ele[1]); /* Side */
316  put_bits(pb, 4, pce->num_ele[2]); /* Back */
317  put_bits(pb, 2, pce->num_ele[3]); /* LFE */
318  put_bits(pb, 3, 0); /* Assoc data */
319  put_bits(pb, 4, 0); /* CCs */
320 
321  put_bits(pb, 1, 0); /* Stereo mixdown */
322  put_bits(pb, 1, 0); /* Mono mixdown */
323  put_bits(pb, 1, 0); /* Something else */
324 
325  for (i = 0; i < 4; i++) {
326  for (j = 0; j < pce->num_ele[i]; j++) {
327  if (i < 3)
328  put_bits(pb, 1, pce->pairing[i][j]);
329  put_bits(pb, 4, pce->index[i][j]);
330  }
331  }
332 
333  align_put_bits(pb);
334  put_bits(pb, 8, strlen(aux_data));
335  ff_put_string(pb, aux_data, 0);
336 }
337 
338 /**
339  * Make AAC audio config object.
340  * @see 1.6.2.1 "Syntax - AudioSpecificConfig"
341  */
342 static int put_audio_specific_config(AVCodecContext *avctx, int chcfg)
343 {
344  PutBitContext pb;
345  AACEncContext *s = avctx->priv_data;
346  const int max_size = 32;
347 
348  avctx->extradata = av_mallocz(max_size);
349  if (!avctx->extradata)
350  return AVERROR(ENOMEM);
351 
352  init_put_bits(&pb, avctx->extradata, max_size);
353  put_bits(&pb, 5, s->profile+1); //profile
354  put_bits(&pb, 4, s->samplerate_index); //sample rate index
355  put_bits(&pb, 4, chcfg);
356  //GASpecificConfig
357  put_bits(&pb, 1, 0); //frame length - 1024 samples
358  put_bits(&pb, 1, 0); //does not depend on core coder
359  put_bits(&pb, 1, 0); //is not extension
360  if (s->needs_pce)
361  put_pce(&pb, avctx);
362 
363  //Explicitly Mark SBR absent
364  put_bits(&pb, 11, 0x2b7); //sync extension
365  put_bits(&pb, 5, AOT_SBR);
366  put_bits(&pb, 1, 0);
367  flush_put_bits(&pb);
368  avctx->extradata_size = put_bytes_output(&pb);
369 
370  return 0;
371 }
372 
374 {
375  ++s->quantize_band_cost_cache_generation;
376  if (s->quantize_band_cost_cache_generation == 0) {
377  memset(s->quantize_band_cost_cache, 0, sizeof(s->quantize_band_cost_cache));
378  s->quantize_band_cost_cache_generation = 1;
379  }
380 }
381 
382 #define WINDOW_FUNC(type) \
383 static void apply_ ##type ##_window(AVFloatDSPContext *fdsp, \
384  SingleChannelElement *sce, \
385  const float *audio)
386 
387 WINDOW_FUNC(only_long)
388 {
389  const float *lwindow = sce->ics.use_kb_window[0] ? ff_aac_kbd_long_1024 : ff_sine_1024;
390  const float *pwindow = sce->ics.use_kb_window[1] ? ff_aac_kbd_long_1024 : ff_sine_1024;
391  float *out = sce->ret_buf;
392 
393  fdsp->vector_fmul (out, audio, lwindow, 1024);
394  fdsp->vector_fmul_reverse(out + 1024, audio + 1024, pwindow, 1024);
395 }
396 
397 WINDOW_FUNC(long_start)
398 {
399  const float *lwindow = sce->ics.use_kb_window[1] ? ff_aac_kbd_long_1024 : ff_sine_1024;
400  const float *swindow = sce->ics.use_kb_window[0] ? ff_aac_kbd_short_128 : ff_sine_128;
401  float *out = sce->ret_buf;
402 
403  fdsp->vector_fmul(out, audio, lwindow, 1024);
404  memcpy(out + 1024, audio + 1024, sizeof(out[0]) * 448);
405  fdsp->vector_fmul_reverse(out + 1024 + 448, audio + 1024 + 448, swindow, 128);
406  memset(out + 1024 + 576, 0, sizeof(out[0]) * 448);
407 }
408 
409 WINDOW_FUNC(long_stop)
410 {
411  const float *lwindow = sce->ics.use_kb_window[0] ? ff_aac_kbd_long_1024 : ff_sine_1024;
412  const float *swindow = sce->ics.use_kb_window[1] ? ff_aac_kbd_short_128 : ff_sine_128;
413  float *out = sce->ret_buf;
414 
415  memset(out, 0, sizeof(out[0]) * 448);
416  fdsp->vector_fmul(out + 448, audio + 448, swindow, 128);
417  memcpy(out + 576, audio + 576, sizeof(out[0]) * 448);
418  fdsp->vector_fmul_reverse(out + 1024, audio + 1024, lwindow, 1024);
419 }
420 
421 WINDOW_FUNC(eight_short)
422 {
423  const float *swindow = sce->ics.use_kb_window[0] ? ff_aac_kbd_short_128 : ff_sine_128;
424  const float *pwindow = sce->ics.use_kb_window[1] ? ff_aac_kbd_short_128 : ff_sine_128;
425  const float *in = audio + 448;
426  float *out = sce->ret_buf;
427  int w;
428 
429  for (w = 0; w < 8; w++) {
430  fdsp->vector_fmul (out, in, w ? pwindow : swindow, 128);
431  out += 128;
432  in += 128;
433  fdsp->vector_fmul_reverse(out, in, swindow, 128);
434  out += 128;
435  }
436 }
437 
438 static void (*const apply_window[4])(AVFloatDSPContext *fdsp,
440  const float *audio) = {
441  [ONLY_LONG_SEQUENCE] = apply_only_long_window,
442  [LONG_START_SEQUENCE] = apply_long_start_window,
443  [EIGHT_SHORT_SEQUENCE] = apply_eight_short_window,
444  [LONG_STOP_SEQUENCE] = apply_long_stop_window
445 };
446 
448  float *audio)
449 {
450  int i;
451  float *output = sce->ret_buf;
452 
453  apply_window[sce->ics.window_sequence[0]](s->fdsp, sce, audio);
454 
456  s->mdct1024_fn(s->mdct1024, sce->coeffs, output, sizeof(float));
457  else
458  for (i = 0; i < 1024; i += 128)
459  s->mdct128_fn(s->mdct128, &sce->coeffs[i], output + i*2, sizeof(float));
460  memcpy(audio, audio + 1024, sizeof(audio[0]) * 1024);
461  memcpy(sce->pcoeffs, sce->coeffs, sizeof(sce->pcoeffs));
462 }
463 
464 /**
465  * Encode ics_info element.
466  * @see Table 4.6 (syntax of ics_info)
467  */
469 {
470  int w;
471 
472  put_bits(&s->pb, 1, 0); // ics_reserved bit
473  put_bits(&s->pb, 2, info->window_sequence[0]);
474  put_bits(&s->pb, 1, info->use_kb_window[0]);
475  if (info->window_sequence[0] != EIGHT_SHORT_SEQUENCE) {
476  put_bits(&s->pb, 6, info->max_sfb);
477  put_bits(&s->pb, 1, 0); /* No predictor present */
478  } else {
479  put_bits(&s->pb, 4, info->max_sfb);
480  for (w = 1; w < 8; w++)
481  put_bits(&s->pb, 1, !info->group_len[w]);
482  }
483 }
484 
485 /**
486  * Encode MS data.
487  * @see 4.6.8.1 "Joint Coding - M/S Stereo"
488  */
490 {
491  int i, w;
492 
493  put_bits(pb, 2, cpe->ms_mode);
494  if (cpe->ms_mode == 1)
495  for (w = 0; w < cpe->ch[0].ics.num_windows; w += cpe->ch[0].ics.group_len[w])
496  for (i = 0; i < cpe->ch[0].ics.max_sfb; i++)
497  put_bits(pb, 1, cpe->ms_mask[w*16 + i]);
498 }
499 
500 /**
501  * Produce integer coefficients from scalefactors provided by the model.
502  */
503 static void adjust_frame_information(ChannelElement *cpe, int chans)
504 {
505  int i, w, w2, g, ch;
506  int maxsfb, cmaxsfb;
507 
508  for (ch = 0; ch < chans; ch++) {
509  IndividualChannelStream *ics = &cpe->ch[ch].ics;
510  maxsfb = 0;
511  cpe->ch[ch].pulse.num_pulse = 0;
512  for (w = 0; w < ics->num_windows; w += ics->group_len[w]) {
513  for (cmaxsfb = ics->num_swb; cmaxsfb > 0 && cpe->ch[ch].zeroes[w*16+cmaxsfb-1]; cmaxsfb--)
514  ;
515  maxsfb = FFMAX(maxsfb, cmaxsfb);
516  }
517  ics->max_sfb = maxsfb;
518 
519  //adjust zero bands for window groups
520  for (w = 0; w < ics->num_windows; w += ics->group_len[w]) {
521  for (g = 0; g < ics->max_sfb; g++) {
522  i = 1;
523  for (w2 = w; w2 < w + ics->group_len[w]; w2++) {
524  if (!cpe->ch[ch].zeroes[w2*16 + g]) {
525  i = 0;
526  break;
527  }
528  }
529  cpe->ch[ch].zeroes[w*16 + g] = i;
530  }
531  }
532  }
533 
534  if (chans > 1 && cpe->common_window) {
535  IndividualChannelStream *ics0 = &cpe->ch[0].ics;
536  IndividualChannelStream *ics1 = &cpe->ch[1].ics;
537  int msc = 0;
538  ics0->max_sfb = FFMAX(ics0->max_sfb, ics1->max_sfb);
539  ics1->max_sfb = ics0->max_sfb;
540  for (w = 0; w < ics0->num_windows*16; w += 16)
541  for (i = 0; i < ics0->max_sfb; i++)
542  if (cpe->ms_mask[w+i])
543  msc++;
544  if (msc == 0 || ics0->max_sfb == 0)
545  cpe->ms_mode = 0;
546  else
547  cpe->ms_mode = msc < ics0->max_sfb * ics0->num_windows ? 1 : 2;
548  }
549 }
550 
552 {
553  int w, w2, g, i;
554  IndividualChannelStream *ics = &cpe->ch[0].ics;
555  if (!cpe->common_window)
556  return;
557  for (w = 0; w < ics->num_windows; w += ics->group_len[w]) {
558  for (w2 = 0; w2 < ics->group_len[w]; w2++) {
559  int start = (w+w2) * 128;
560  for (g = 0; g < ics->num_swb; g++) {
561  int p = -1 + 2 * (cpe->ch[1].band_type[w*16+g] - 14);
562  float scale = cpe->ch[0].is_ener[w*16+g];
563  if (!cpe->is_mask[w*16 + g]) {
564  start += ics->swb_sizes[g];
565  continue;
566  }
567  if (cpe->ms_mask[w*16 + g])
568  p *= -1;
569  for (i = 0; i < ics->swb_sizes[g]; i++) {
570  float sum = (cpe->ch[0].coeffs[start+i] + p*cpe->ch[1].coeffs[start+i])*scale;
571  cpe->ch[0].coeffs[start+i] = sum;
572  cpe->ch[1].coeffs[start+i] = 0.0f;
573  }
574  start += ics->swb_sizes[g];
575  }
576  }
577  }
578 }
579 
580 /* Intensity stereo is only allowed when its irreducible image error */
581 #define NMR_IS_IMG_GATE 0.5f
582 
583 /* Frequency in Hz for the lower limit of intensity stereo */
584 #define NMR_IS_LOW_LIMIT 6100
585 
586 /* Rate ceiling (bits/sample/channel) above which intensity is skipped, ~145kbps */
587 #define NMR_IS_MAXBPS 1.52f
588 
589 /* The rate ceiling is lifted on hard-to-code frames. The signal is the bit
590  * reservoir going into deficit: a negative fill means the trellis is spending
591  * more than the nominal rate to hold quality (operating lambda has climbed). */
592 #define NMR_IS_FILLGAIN 0.27f
593 #define NMR_IS_FILLMAX 0.40f
594 
595 /* M/S thresholds: a band is recoded as mid+side when the side is negligible */
596 #define NMR_MS_EQUIV 0.01f
597 #define NMR_MS_MASK 0.0f
598 
599 /* PNS-stereo decorrelation gate: a band may be noise-substituted in a CPE only if its
600  * side energy is at least this fraction of its mid energy, i.e. the image is genuinely
601  * wide (channels decorrelated). PNS renders uncorrelated noise per channel, so it only
602  * preserves the image on already-wide bands; a much stricter bar than I/S (which can
603  * collapse correlated bands). Lower = more PNS / more imaging risk. */
604 #define NMR_PNS_STEREO_DECORR 0.6f
605 
606 /* Recode one band's window group as mid+side in place, updating the psy band
607  * energies/thresholds to the M/S spectra. The threshold is halved as a coarse guard
608  * against L/R unmasking of the independently-quantized M/S noise (M/S is a lossless
609  * rotation but lossy coding). Used for the M/S decision and the intensity fallback. */
611  int w, int g, int start, int len, int gl)
612 {
613  SingleChannelElement *sce0 = &cpe->ch[0];
614  SingleChannelElement *sce1 = &cpe->ch[1];
615  cpe->ms_mask[w*16+g] = 1;
616  for (int w2 = 0; w2 < gl; w2++) {
617  FFPsyBand *b0 = &s->psy.ch[s->cur_channel+0].psy_bands[(w+w2)*16+g];
618  FFPsyBand *b1 = &s->psy.ch[s->cur_channel+1].psy_bands[(w+w2)*16+g];
619  float *L = sce0->coeffs + start + (w+w2)*128;
620  float *R = sce1->coeffs + start + (w+w2)*128;
621  float em = 0.0f, es = 0.0f;
622  for (int i = 0; i < len; i++) {
623  float m = (L[i] + R[i]) * 0.5f;
624  R[i] = m - R[i]; L[i] = m;
625  em += L[i]*L[i]; es += R[i]*R[i];
626  }
627  b0->threshold = b1->threshold = FFMIN(b0->threshold, b1->threshold) * 0.5f;
628  b0->energy = em; b1->energy = es;
629  }
630 }
631 
632 /* Intensity-stereo perceptual test for one band's window group: collapse the pair
633  * to a single carrier (L + p*R)*scale that the decoder rescales per channel, and
634  * check that the irreducible image error, which no bit budget can reduce, is
635  * masked in both channels. On success returns 1 and fills the carrier scale, the
636  * decoder's R/carrier ratio sr_, and the phase p. The caller restricts this to HF
637  * bands with energy in both channels. */
639  int w, int g, int start, int len, int gl,
640  float ener0, float ener1, float dot,
641  float minthr0, float minthr1,
642  float *scale_out, float *sr_out, int *p_out)
643 {
644  int p = dot >= 0.0f ? 1 : -1;
645  float ener01 = ener0 + ener1 + 2*p*dot; /* energy of L + p*R */
646  if (ener01 <= FLT_MIN)
647  return 0;
648  float scale = sqrtf(ener0 / ener01); /* carrier = (L + p*R)*scale */
649  float sr_ = sqrtf(ener1 / ener0); /* decoder: R = p*sr_*carrier */
650  float img0 = 0.0f, img1 = 0.0f;
651  for (int w2 = 0; w2 < gl; w2++) {
652  const float *L = cpe->ch[0].coeffs + start + (w+w2)*128;
653  const float *R = cpe->ch[1].coeffs + start + (w+w2)*128;
654  for (int i = 0; i < len; i++) {
655  float c = (L[i] + p*R[i]) * scale;
656  float dl = L[i] - c, dr = R[i] - p*sr_*c;
657  img0 += dl*dl; img1 += dr*dr;
658  }
659  }
660  if (img0 >= NMR_IS_IMG_GATE * minthr0 * gl ||
661  img1 >= NMR_IS_IMG_GATE * minthr1 * gl)
662  return 0;
663  *scale_out = scale; *sr_out = sr_; *p_out = p;
664  return 1;
665 }
666 
667 /* Recode one band's window group as intensity stereo in place: replace L with the
668  * carrier, zero R, signal the phase via the side channel's band type, and fold the
669  * pair's masking into the surviving (carrier) channel. */
671  int w, int g, int start, int len, int gl,
672  float scale, float sr_, int p,
673  float ener0, float ener1)
674 {
675  cpe->is_mask[w*16+g] = 1;
676  cpe->ch[0].is_ener[w*16+g] = scale;
677  cpe->ch[1].is_ener[w*16+g] = ener0 / ener1;
678  cpe->ch[1].band_type[w*16+g] = p > 0 ? INTENSITY_BT : INTENSITY_BT2;
679  for (int w2 = 0; w2 < gl; w2++) {
680  FFPsyBand *b0 = &s->psy.ch[s->cur_channel+0].psy_bands[(w+w2)*16+g];
681  FFPsyBand *b1 = &s->psy.ch[s->cur_channel+1].psy_bands[(w+w2)*16+g];
682  float *L = cpe->ch[0].coeffs + start + (w+w2)*128;
683  float *R = cpe->ch[1].coeffs + start + (w+w2)*128;
684  float ec = 0.0f;
685  for (int i = 0; i < len; i++) {
686  L[i] = (L[i] + p*R[i]) * scale;
687  R[i] = 0.0f;
688  ec += L[i]*L[i];
689  }
690  b0->threshold = FFMIN(b0->threshold, b1->threshold / FFMAX(sr_*sr_, 1e-9f));
691  b0->energy = ec; b1->energy = 0.0f;
692  }
693 }
694 
695 /*
696  * Per-band stereo-mode decision (L/R vs M/S vs intensity) for the NMR coder,
697  * made before quantization from the psychoacoustic model alone, so the
698  * quantizer search allocates natively on the spectra that are actually coded.
699  */
701 {
702  SingleChannelElement *sce0 = &cpe->ch[0];
703  SingleChannelElement *sce1 = &cpe->ch[1];
704  IndividualChannelStream *ics = &sce0->ics;
705  const AVCodecContext *avctx = s->psy.avctx;
706  const float freq_mult = avctx->sample_rate / (1024.0f / ics->num_windows) / 2.0f;
707  const float bps = avctx->bit_rate > 0 ?
708  (float)avctx->bit_rate / avctx->sample_rate / avctx->ch_layout.nb_channels : 0.0f;
709  int is_count = 0;
710 
711  /* Stereo decision, with no bitrate dependence. Start from full L/R and depart from
712  * it only where the change is inaudible. M/S and I/S differ in what they trade:
713  * M/S recodes the pair as mid+side -- an invertible rotation, but the M and S
714  * are quantized independently, so it is lossy coding whose noise un-mixes
715  * back to L/R. Used where it barely changes the result (the side is
716  * negligible vs the mid, so it is ~equivalent to L/R at the same rate) --
717  * OR where the doubled side energy is masked.
718  * I/S drops the side phase and keeps its energy, where the residual image error
719  * is masked. Used for the decorrelated HF that M/S cannot help.
720  * Both tests are content/perceptual and frame-stable, so the image holds. */
721 
722  /* I/S rate gate: eligible at/below ~128 kbps, with the ceiling lifted on hard
723  * frames (bit reservoir in deficit) so a starved high-rate passage can still
724  * call on intensity. Where an I/S candidate is found but IS is not eligible, fall
725  * back to M/S: not free, but ~equivalent to L/R there and it lets the energy
726  * compact into the mid. */
727  const float rate_frame = avctx->bit_rate * 1024.0f / FFMAX(avctx->sample_rate, 1);
728  const float deficit = (s->nmr && rate_frame > 0.0f)
729  ? FFMAX(0.0f, -(float)s->nmr->rc_fill / rate_frame) : 0.0f;
730  const float is_bonus = FFMIN(NMR_IS_FILLMAX, NMR_IS_FILLGAIN * deficit);
731  const int allow_is = s->options.intensity_stereo && bps < NMR_IS_MAXBPS + is_bonus;
732 
733  for (int w = 0; w < ics->num_windows; w += ics->group_len[w]) {
734  int start = 0;
735  for (int g = 0; g < ics->num_swb; start += ics->swb_sizes[g++]) {
736  int len = ics->swb_sizes[g], gl = ics->group_len[w];
737  float ener0 = 0.0f, ener1 = 0.0f, dot = 0.0f, es_tot = 0.0f, em_tot = 0.0f;
738  float minthr0 = FLT_MAX, minthr1 = FLT_MAX;
739 
740  cpe->is_mask[w*16+g] = 0;
741  cpe->ms_mask[w*16+g] = 0;
742 
743  for (int w2 = 0; w2 < gl; w2++) {
744  FFPsyBand *b0 = &s->psy.ch[s->cur_channel+0].psy_bands[(w+w2)*16+g];
745  FFPsyBand *b1 = &s->psy.ch[s->cur_channel+1].psy_bands[(w+w2)*16+g];
746  const float *L = sce0->coeffs + start + (w+w2)*128;
747  const float *R = sce1->coeffs + start + (w+w2)*128;
748  float el = 0.0f, er = 0.0f, em = 0.0f, es = 0.0f, d = 0.0f;
749  for (int i = 0; i < len; i++) {
750  float m = (L[i] + R[i]) * 0.5f;
751  float sv = m - R[i];
752  el += L[i]*L[i]; er += R[i]*R[i];
753  em += m*m; es += sv*sv; d += L[i]*R[i];
754  }
755  ener0 += el; ener1 += er; dot += d; es_tot += es; em_tot += em;
756  minthr0 = FFMIN(minthr0, b0->threshold);
757  minthr1 = FFMIN(minthr1, b1->threshold);
758  }
759  float thr_g = FFMIN(minthr0, minthr1) * gl; /* group masking budget */
760 
761  /* PNS-stereo reservation. Reserve a band for noise substitution only if it
762  * is noise-like in both channels (intersected can_pns) and clearly
763  * decorrelated (wide image). */
764  if (cpe->ch[0].can_pns[w*16+g] && cpe->ch[1].can_pns[w*16+g] &&
765  es_tot > NMR_PNS_STEREO_DECORR * em_tot)
766  continue;
767  cpe->ch[0].can_pns[w*16+g] = cpe->ch[1].can_pns[w*16+g] = 0;
768 
769  int ms_ok = s->options.mid_side &&
770  (s->options.mid_side == 1 ||
771  es_tot < NMR_MS_EQUIV * em_tot ||
772  es_tot < NMR_MS_MASK * thr_g);
773  float scale, sr_; int p;
774  int is_ok = !ms_ok &&
775  start * freq_mult > NMR_IS_LOW_LIMIT &&
776  ener0 > FLT_MIN && ener1 > FLT_MIN &&
777  nmr_is_image_masked(s, cpe, w, g, start, len, gl,
778  ener0, ener1, dot, minthr0, minthr1,
779  &scale, &sr_, &p);
780 
781  if (ms_ok) {
782  nmr_apply_ms_band(s, cpe, w, g, start, len, gl);
783  } else if (is_ok && allow_is) {
784  nmr_apply_is_band(s, cpe, w, g, start, len, gl,
785  scale, sr_, p, ener0, ener1);
786  is_count++;
787  } else if (is_ok && s->options.mid_side) {
788  nmr_apply_ms_band(s, cpe, w, g, start, len, gl);
789  }
790  /* else: keep full L/R stereo */
791  }
792  }
793  cpe->is_mode = !!is_count;
794 }
795 
797 {
798  int w, w2, g, i;
799  IndividualChannelStream *ics = &cpe->ch[0].ics;
800  if (!cpe->common_window)
801  return;
802  for (w = 0; w < ics->num_windows; w += ics->group_len[w]) {
803  for (w2 = 0; w2 < ics->group_len[w]; w2++) {
804  int start = (w+w2) * 128;
805  for (g = 0; g < ics->num_swb; g++) {
806  /* ms_mask can be used for other purposes in PNS and I/S,
807  * so must not apply M/S if any band uses either, even if
808  * ms_mask is set.
809  */
810  if (!cpe->ms_mask[w*16 + g] || cpe->is_mask[w*16 + g]
811  || cpe->ch[0].band_type[w*16 + g] >= NOISE_BT
812  || cpe->ch[1].band_type[w*16 + g] >= NOISE_BT) {
813  start += ics->swb_sizes[g];
814  continue;
815  }
816  for (i = 0; i < ics->swb_sizes[g]; i++) {
817  float L = (cpe->ch[0].coeffs[start+i] + cpe->ch[1].coeffs[start+i]) * 0.5f;
818  float R = L - cpe->ch[1].coeffs[start+i];
819  cpe->ch[0].coeffs[start+i] = L;
820  cpe->ch[1].coeffs[start+i] = R;
821  }
822  start += ics->swb_sizes[g];
823  }
824  }
825  }
826 }
827 
828 /**
829  * Encode scalefactor band coding type.
830  */
832 {
833  int w;
834 
835  if (s->coder->set_special_band_scalefactors)
836  s->coder->set_special_band_scalefactors(s, sce);
837 
838  for (w = 0; w < sce->ics.num_windows; w += sce->ics.group_len[w])
839  s->coder->encode_window_bands_info(s, sce, w, sce->ics.group_len[w], s->lambda);
840 }
841 
842 /**
843  * Encode scalefactors.
844  */
847 {
848  int diff, off_sf = sce->sf_idx[0], off_pns = sce->sf_idx[0] - NOISE_OFFSET;
849  int off_is = 0, noise_flag = 1;
850  int i, w;
851 
852  for (w = 0; w < sce->ics.num_windows; w += sce->ics.group_len[w]) {
853  for (i = 0; i < sce->ics.max_sfb; i++) {
854  if (!sce->zeroes[w*16 + i]) {
855  if (sce->band_type[w*16 + i] == NOISE_BT) {
856  diff = sce->sf_idx[w*16 + i] - off_pns;
857  off_pns = sce->sf_idx[w*16 + i];
858  if (noise_flag-- > 0) {
860  continue;
861  }
862  } else if (sce->band_type[w*16 + i] == INTENSITY_BT ||
863  sce->band_type[w*16 + i] == INTENSITY_BT2) {
864  diff = sce->sf_idx[w*16 + i] - off_is;
865  off_is = sce->sf_idx[w*16 + i];
866  } else {
867  diff = sce->sf_idx[w*16 + i] - off_sf;
868  off_sf = sce->sf_idx[w*16 + i];
869  }
871  av_assert0(diff >= 0 && diff <= 120);
873  }
874  }
875  }
876 }
877 
878 /**
879  * Encode pulse data.
880  */
881 static void encode_pulses(AACEncContext *s, Pulse *pulse)
882 {
883  int i;
884 
885  put_bits(&s->pb, 1, !!pulse->num_pulse);
886  if (!pulse->num_pulse)
887  return;
888 
889  put_bits(&s->pb, 2, pulse->num_pulse - 1);
890  put_bits(&s->pb, 6, pulse->start);
891  for (i = 0; i < pulse->num_pulse; i++) {
892  put_bits(&s->pb, 5, pulse->pos[i]);
893  put_bits(&s->pb, 4, pulse->amp[i]);
894  }
895 }
896 
897 /**
898  * Encode spectral coefficients processed by psychoacoustic model.
899  */
901 {
902  int start, i, w, w2;
903 
904  for (w = 0; w < sce->ics.num_windows; w += sce->ics.group_len[w]) {
905  start = 0;
906  for (i = 0; i < sce->ics.max_sfb; i++) {
907  if (sce->zeroes[w*16 + i]) {
908  start += sce->ics.swb_sizes[i];
909  continue;
910  }
911  for (w2 = w; w2 < w + sce->ics.group_len[w]; w2++) {
912  s->coder->quantize_and_encode_band(s, &s->pb,
913  &sce->coeffs[start + w2*128],
914  NULL, sce->ics.swb_sizes[i],
915  sce->sf_idx[w*16 + i],
916  sce->band_type[w*16 + i],
917  s->lambda,
918  sce->ics.window_clipping[w]);
919  }
920  start += sce->ics.swb_sizes[i];
921  }
922  }
923 }
924 
925 /**
926  * Downscale spectral coefficients for near-clipping windows to avoid artifacts
927  */
929 {
930  int start, i, j, w;
931 
932  if (sce->ics.clip_avoidance_factor < 1.0f) {
933  for (w = 0; w < sce->ics.num_windows; w++) {
934  start = 0;
935  for (i = 0; i < sce->ics.max_sfb; i++) {
936  float *swb_coeffs = &sce->coeffs[start + w*128];
937  for (j = 0; j < sce->ics.swb_sizes[i]; j++)
938  swb_coeffs[j] *= sce->ics.clip_avoidance_factor;
939  start += sce->ics.swb_sizes[i];
940  }
941  }
942  }
943 }
944 
945 /**
946  * Encode one channel of audio data.
947  */
950  int common_window)
951 {
952  put_bits(&s->pb, 8, sce->sf_idx[0]);
953  if (!common_window)
954  put_ics_info(s, &sce->ics);
955  encode_band_info(s, sce);
956  encode_scale_factors(avctx, s, sce);
957  encode_pulses(s, &sce->pulse);
958  put_bits(&s->pb, 1, !!sce->tns.present);
959  if (s->coder->encode_tns_info)
960  s->coder->encode_tns_info(s, sce);
961  put_bits(&s->pb, 1, 0); //ssr
963  return 0;
964 }
965 
966 /**
967  * Write some auxiliary information about the created AAC file.
968  */
969 static void put_bitstream_info(AACEncContext *s, const char *name)
970 {
971  int i, namelen, padbits;
972 
973  namelen = strlen(name) + 2;
974  put_bits(&s->pb, 3, TYPE_FIL);
975  put_bits(&s->pb, 4, FFMIN(namelen, 15));
976  if (namelen >= 15)
977  put_bits(&s->pb, 8, namelen - 14);
978  put_bits(&s->pb, 4, 0); //extension type - filler
979  padbits = -put_bits_count(&s->pb) & 7;
980  align_put_bits(&s->pb);
981  for (i = 0; i < namelen - 2; i++)
982  put_bits(&s->pb, 8, name[i]);
983  put_bits(&s->pb, 12 - padbits, 0);
984 }
985 
986 /*
987  * Copy input samples.
988  * Channels are reordered from libavcodec's default order to AAC order.
989  */
991 {
992  int ch;
993  int end = 2048 + (frame ? frame->nb_samples : 0);
994  const uint8_t *channel_map = s->reorder_map;
995 
996  /* copy and remap input samples */
997  for (ch = 0; ch < s->channels; ch++) {
998  /* copy last 1024 samples of previous frame to the start of the current frame */
999  memcpy(&s->planar_samples[ch][1024], &s->planar_samples[ch][2048], 1024 * sizeof(s->planar_samples[0][0]));
1000 
1001  /* copy new samples and zero any remaining samples */
1002  if (frame) {
1003  memcpy(&s->planar_samples[ch][2048],
1004  frame->extended_data[channel_map[ch]],
1005  frame->nb_samples * sizeof(s->planar_samples[0][0]));
1006  }
1007  memset(&s->planar_samples[ch][end], 0,
1008  (3072 - end) * sizeof(s->planar_samples[0][0]));
1009  }
1010 }
1011 
1012 static int aac_encode_frame(AVCodecContext *avctx, AVPacket *avpkt,
1013  const AVFrame *frame, int *got_packet_ptr)
1014 {
1015  AACEncContext *s = avctx->priv_data;
1016  float **samples = s->planar_samples, *samples2, *la, *overlap;
1017  ChannelElement *cpe;
1018  SingleChannelElement *sce;
1020  int i, its, ch, w, chans, tag, start_ch, ret, frame_bits;
1021  int target_bits, rate_bits, too_many_bits, too_few_bits;
1022  int ms_mode = 0, is_mode = 0, tns_mode = 0, pred_mode = 0;
1023  int chan_el_counter[4];
1025 
1026  /* add current frame to queue */
1027  if (frame) {
1028  if ((ret = ff_af_queue_add(&s->afq, frame)) < 0)
1029  return ret;
1030  } else {
1031  if (!s->afq.remaining_samples || (!s->afq.frame_alloc && !s->afq.frame_count))
1032  return 0;
1033  }
1034 
1036 
1037  if (!avctx->frame_num)
1038  return 0;
1039 
1040  start_ch = 0;
1041  for (i = 0; i < s->chan_map[0]; i++) {
1042  FFPsyWindowInfo* wi = windows + start_ch;
1043  tag = s->chan_map[i+1];
1044  chans = tag == TYPE_CPE ? 2 : 1;
1045  cpe = &s->cpe[i];
1046  for (ch = 0; ch < chans; ch++) {
1047  int k;
1048  float clip_avoidance_factor;
1049  sce = &cpe->ch[ch];
1050  ics = &sce->ics;
1051  s->cur_channel = start_ch + ch;
1052  overlap = &samples[s->cur_channel][0];
1053  samples2 = overlap + 1024;
1054  la = samples2 + (448+64);
1055  if (!frame)
1056  la = NULL;
1057  if (tag == TYPE_LFE) {
1058  wi[ch].window_type[0] = wi[ch].window_type[1] = ONLY_LONG_SEQUENCE;
1059  wi[ch].window_shape = 0;
1060  wi[ch].num_windows = 1;
1061  wi[ch].grouping[0] = 1;
1062  wi[ch].clipping[0] = 0;
1063 
1064  /* Only the lowest 12 coefficients are used in a LFE channel.
1065  * The expression below results in only the bottom 8 coefficients
1066  * being used for 11.025kHz to 16kHz sample rates.
1067  */
1068  ics->num_swb = s->samplerate_index >= 8 ? 1 : 3;
1069  } else {
1070  wi[ch] = s->psy.model->window(&s->psy, samples2, la, s->cur_channel,
1071  ics->window_sequence[0]);
1072  }
1073  ics->window_sequence[1] = ics->window_sequence[0];
1074  ics->window_sequence[0] = wi[ch].window_type[0];
1075  ics->use_kb_window[1] = ics->use_kb_window[0];
1076  ics->use_kb_window[0] = wi[ch].window_shape;
1077  ics->num_windows = wi[ch].num_windows;
1078  ics->swb_sizes = s->psy.bands [ics->num_windows == 8];
1079  ics->num_swb = tag == TYPE_LFE ? ics->num_swb : s->psy.num_bands[ics->num_windows == 8];
1080  ics->max_sfb = FFMIN(ics->max_sfb, ics->num_swb);
1081  ics->swb_offset = wi[ch].window_type[0] == EIGHT_SHORT_SEQUENCE ?
1082  ff_swb_offset_128 [s->samplerate_index]:
1083  ff_swb_offset_1024[s->samplerate_index];
1084  ics->tns_max_bands = wi[ch].window_type[0] == EIGHT_SHORT_SEQUENCE ?
1085  ff_tns_max_bands_128 [s->samplerate_index]:
1086  ff_tns_max_bands_1024[s->samplerate_index];
1087 
1088  for (w = 0; w < ics->num_windows; w++)
1089  ics->group_len[w] = wi[ch].grouping[w];
1090 
1091  /* Calculate input sample maximums and evaluate clipping risk */
1092  clip_avoidance_factor = 0.0f;
1093  for (w = 0; w < ics->num_windows; w++) {
1094  const float *wbuf = overlap + w * 128;
1095  const int wlen = 2048 / ics->num_windows;
1096  float max = 0;
1097  int j;
1098  /* mdct input is 2 * output */
1099  for (j = 0; j < wlen; j++)
1100  max = FFMAX(max, fabsf(wbuf[j]));
1101  wi[ch].clipping[w] = max;
1102  }
1103  for (w = 0; w < ics->num_windows; w++) {
1104  if (wi[ch].clipping[w] > CLIP_AVOIDANCE_FACTOR) {
1105  ics->window_clipping[w] = 1;
1106  clip_avoidance_factor = FFMAX(clip_avoidance_factor, wi[ch].clipping[w]);
1107  } else {
1108  ics->window_clipping[w] = 0;
1109  }
1110  }
1111  if (clip_avoidance_factor > CLIP_AVOIDANCE_FACTOR) {
1112  ics->clip_avoidance_factor = CLIP_AVOIDANCE_FACTOR / clip_avoidance_factor;
1113  } else {
1114  ics->clip_avoidance_factor = 1.0f;
1115  }
1116 
1117  apply_window_and_mdct(s, sce, overlap);
1118 
1119  for (k = 0; k < 1024; k++) {
1120  if (!(fabs(cpe->ch[ch].coeffs[k]) < 1E16)) { // Ensure headroom for energy calculation
1121  av_log(avctx, AV_LOG_ERROR, "Input contains (near) NaN/+-Inf\n");
1122  return AVERROR(EINVAL);
1123  }
1124  }
1125  avoid_clipping(s, sce);
1126  }
1127  start_ch += chans;
1128  }
1129  if ((ret = ff_alloc_packet(avctx, avpkt, 8192 * s->channels)) < 0)
1130  return ret;
1131  frame_bits = its = 0;
1132  do {
1133  init_put_bits(&s->pb, avpkt->data, avpkt->size);
1134 
1135  if ((avctx->frame_num & 0xFF)==1 && !(avctx->flags & AV_CODEC_FLAG_BITEXACT))
1137  start_ch = 0;
1138  target_bits = 0;
1139  memset(chan_el_counter, 0, sizeof(chan_el_counter));
1140  for (i = 0; i < s->chan_map[0]; i++) {
1141  FFPsyWindowInfo* wi = windows + start_ch;
1142  const float *coeffs[2];
1143  tag = s->chan_map[i+1];
1144  chans = tag == TYPE_CPE ? 2 : 1;
1145  cpe = &s->cpe[i];
1146  cpe->common_window = 0;
1147  memset(cpe->is_mask, 0, sizeof(cpe->is_mask));
1148  memset(cpe->ms_mask, 0, sizeof(cpe->ms_mask));
1149  put_bits(&s->pb, 3, tag);
1150  put_bits(&s->pb, 4, chan_el_counter[tag]++);
1151  for (ch = 0; ch < chans; ch++) {
1152  sce = &cpe->ch[ch];
1153  coeffs[ch] = sce->coeffs;
1154  memset(&sce->tns, 0, sizeof(TemporalNoiseShaping));
1155  for (w = 0; w < 128; w++)
1156  if (sce->band_type[w] > RESERVED_BT)
1157  sce->band_type[w] = 0;
1158  }
1159  s->psy.bitres.alloc = -1;
1160  s->psy.bitres.bits = s->last_frame_pb_count / s->channels;
1161  s->psy.model->analyze(&s->psy, start_ch, coeffs, wi);
1162  if (s->psy.bitres.alloc > 0) {
1163  /* Lambda unused here on purpose, we need to take psy's unscaled allocation */
1164  target_bits += s->psy.bitres.alloc
1165  * (s->lambda / (avctx->global_quality ? avctx->global_quality : 120));
1166  s->psy.bitres.alloc /= chans;
1167  }
1168  s->cur_type = tag;
1169  if (chans > 1
1170  && wi[0].window_type[0] == wi[1].window_type[0]
1171  && wi[0].window_shape == wi[1].window_shape) {
1172 
1173  cpe->common_window = 1;
1174  for (w = 0; w < wi[0].num_windows; w++) {
1175  if (wi[0].grouping[w] != wi[1].grouping[w]) {
1176  cpe->common_window = 0;
1177  break;
1178  }
1179  }
1180  }
1181 
1182  const int use_tns = s->options.tns && s->coder->search_for_tns &&
1183  s->coder->apply_tns_filt;
1184 
1185  /* The NMR coder rate-controls itself and never re-quantizes, so TNS must run
1186  * before the quantizer */
1187  const int tns_first = s->options.coder == AAC_CODER_NMR;
1188  if (tns_first && use_tns) {
1189  for (ch = 0; ch < chans; ch++) {
1190  sce = &cpe->ch[ch];
1191  s->cur_channel = start_ch + ch;
1192  /* mono: mark_pns before TNS so the region cap sees PNS bands. Stereo
1193  * PNS is marked in its own block (below) after the stereo decision. */
1194  if (chans == 1 && s->options.pns && s->coder->mark_pns)
1195  s->coder->mark_pns(s, avctx, sce);
1196  s->coder->search_for_tns(s, sce);
1197  s->coder->apply_tns_filt(s, sce);
1198  if (sce->tns.present)
1199  tns_mode = 1;
1200  }
1201  }
1202 
1203  /* NMR stereo PNS (imaging-safe). Mark each channel's noise-like bands on the
1204  * original L/R psy, then keep PNS only where BOTH channels are noise-like. */
1205  if (chans == 2 && cpe->common_window && tns_first &&
1206  s->options.pns && s->coder->mark_pns) {
1207  s->cur_channel = start_ch; s->coder->mark_pns(s, avctx, &cpe->ch[0]);
1208  s->cur_channel = start_ch + 1; s->coder->mark_pns(s, avctx, &cpe->ch[1]);
1209  for (int b = 0; b < 128; b++)
1210  if (!cpe->ch[0].can_pns[b] || !cpe->ch[1].can_pns[b])
1211  cpe->ch[0].can_pns[b] = cpe->ch[1].can_pns[b] = 0;
1212  }
1213 
1214  /* The NMR coder decides I/S and M/S BEFORE quantization, from the psy model,
1215  * and the trellis then allocates natively on the coeffs actually coded. */
1216  if (chans == 2 && cpe->common_window && s->options.coder == AAC_CODER_NMR &&
1217  (s->options.mid_side || s->options.intensity_stereo)) {
1218  s->cur_channel = start_ch;
1219  nmr_decide_stereo(s, cpe);
1220  }
1221  for (ch = 0; ch < chans; ch++) {
1222  s->cur_channel = start_ch + ch;
1223  /* NMR PNS is mono-only */
1224  if (s->options.pns && s->coder->mark_pns && !tns_first)
1225  s->coder->mark_pns(s, avctx, &cpe->ch[ch]);
1226  s->coder->search_for_quantizers(avctx, s, &cpe->ch[ch], s->lambda);
1227  }
1228  for (ch = 0; ch < chans; ch++) { /* TNS (non-NMR) and PNS */
1229  sce = &cpe->ch[ch];
1230  s->cur_channel = start_ch + ch;
1231  if (!tns_first && use_tns) {
1232  s->coder->search_for_tns(s, sce);
1233  s->coder->apply_tns_filt(s, sce);
1234  if (sce->tns.present)
1235  tns_mode = 1;
1236  }
1237  if (s->options.pns && s->coder->search_for_pns)
1238  s->coder->search_for_pns(s, avctx, sce);
1239  }
1240  s->cur_channel = start_ch;
1241  if (s->options.intensity_stereo) { /* Intensity Stereo */
1242  if (s->options.coder != AAC_CODER_NMR) { /* NMR: decided pre-search */
1243  if (s->coder->search_for_is)
1244  s->coder->search_for_is(s, avctx, cpe);
1246  }
1247  if (cpe->is_mode) is_mode = 1;
1248  }
1249  if (s->options.mid_side && s->options.coder != AAC_CODER_NMR) { /* Mid/Side stereo */
1250  if (s->options.mid_side == -1 && s->coder->search_for_ms)
1251  s->coder->search_for_ms(s, cpe);
1252  else if (cpe->common_window)
1253  memset(cpe->ms_mask, 1, sizeof(cpe->ms_mask));
1254  apply_mid_side_stereo(cpe);
1255  }
1256  adjust_frame_information(cpe, chans);
1257  if (chans == 2) {
1258  put_bits(&s->pb, 1, cpe->common_window);
1259  if (cpe->common_window) {
1260  put_ics_info(s, &cpe->ch[0].ics);
1261  encode_ms_info(&s->pb, cpe);
1262  if (cpe->ms_mode) ms_mode = 1;
1263  }
1264  }
1265  for (ch = 0; ch < chans; ch++) {
1266  s->cur_channel = start_ch + ch;
1267  encode_individual_channel(avctx, s, &cpe->ch[ch], cpe->common_window);
1268  }
1269  start_ch += chans;
1270  }
1271 
1272  if (avctx->flags & AV_CODEC_FLAG_QSCALE) {
1273  /* When using a constant Q-scale, don't mess with lambda */
1274  break;
1275  }
1276 
1277  frame_bits = put_bits_count(&s->pb);
1278 
1279  /* The NMR coder rate-controls itself (global-lambda reservoir servo):
1280  * per-frame bits intentionally float around the nominal rate, so skip
1281  * the lambda rate loop and only intervene on a hard overflow. */
1282  if (s->options.coder == AAC_CODER_NMR && avctx->bit_rate_tolerance != 0 &&
1283  frame_bits < 6144 * s->channels - 3)
1284  break;
1285 
1286  /* rate control stuff
1287  * allow between the nominal bitrate, and what psy's bit reservoir says to target
1288  * but drift towards the nominal bitrate always
1289  */
1290  rate_bits = avctx->bit_rate * 1024 / avctx->sample_rate;
1291  rate_bits = FFMIN(rate_bits, 6144 * s->channels - 3);
1292  too_many_bits = FFMAX(target_bits, rate_bits);
1293  too_many_bits = FFMIN(too_many_bits, 6144 * s->channels - 3);
1294  too_few_bits = FFMIN(FFMAX(rate_bits - rate_bits/4, target_bits), too_many_bits);
1295 
1296  /* When strict bit-rate control is demanded */
1297  if (avctx->bit_rate_tolerance == 0) {
1298  if (rate_bits < frame_bits) {
1299  float ratio = ((float)rate_bits) / frame_bits;
1300  s->lambda *= FFMIN(0.9f, ratio);
1301  continue;
1302  }
1303  /* reset lambda when solution is found */
1304  s->lambda = avctx->global_quality > 0 ? avctx->global_quality : 120;
1305  break;
1306  }
1307 
1308  /* When using ABR, be strict (but only for increasing) */
1309  too_few_bits = too_few_bits - too_few_bits/8;
1310  too_many_bits = too_many_bits + too_many_bits/2;
1311 
1312  if ( its == 0 /* for steady-state Q-scale tracking */
1313  || (its < 5 && (frame_bits < too_few_bits || frame_bits > too_many_bits))
1314  || frame_bits >= 6144 * s->channels - 3 )
1315  {
1316  float ratio = ((float)rate_bits) / frame_bits;
1317 
1318  if (frame_bits >= too_few_bits && frame_bits <= too_many_bits) {
1319  /*
1320  * This path is for steady-state Q-scale tracking
1321  * When frame bits fall within the stable range, we still need to adjust
1322  * lambda to maintain it like so in a stable fashion (large jumps in lambda
1323  * create artifacts and should be avoided), but slowly
1324  */
1325  ratio = sqrtf(sqrtf(ratio));
1326  ratio = av_clipf(ratio, 0.9f, 1.1f);
1327  } else {
1328  /* Not so fast though */
1329  ratio = sqrtf(ratio);
1330  }
1331  s->lambda = av_clipf(s->lambda * ratio, FLT_EPSILON, 65536.f);
1332 
1333  /* Keep iterating if we must reduce and lambda is in the sky */
1334  if (ratio > 0.9f && ratio < 1.1f) {
1335  break;
1336  } else {
1337  if (is_mode || ms_mode || tns_mode || pred_mode) {
1338  for (i = 0; i < s->chan_map[0]; i++) {
1339  // Must restore coeffs
1340  chans = tag == TYPE_CPE ? 2 : 1;
1341  cpe = &s->cpe[i];
1342  for (ch = 0; ch < chans; ch++)
1343  memcpy(cpe->ch[ch].coeffs, cpe->ch[ch].pcoeffs, sizeof(cpe->ch[ch].coeffs));
1344  }
1345  }
1346  its++;
1347  }
1348  } else {
1349  break;
1350  }
1351  } while (1);
1352 
1353  /* tool-usage stats over the final per-band decisions of this frame */
1354  for (i = 0; i < s->chan_map[0]; i++) {
1355  int etag = s->chan_map[i + 1], echans = etag == TYPE_CPE ? 2 : 1;
1356  ChannelElement *ce = &s->cpe[i];
1357  IndividualChannelStream *ics = &ce->ch[0].ics;
1358  for (ch = 0; ch < echans; ch++) { /* per-channel frame stats */
1359  int is_short = ce->ch[ch].ics.window_sequence[0] == EIGHT_SHORT_SEQUENCE;
1360  s->stat_chans++;
1361  if (is_short)
1362  s->stat_short++;
1363  if (ce->ch[ch].tns.present) {
1364  if (is_short) s->stat_tns_short++;
1365  else s->stat_tns_long++;
1366  }
1367  }
1368  for (w = 0; w < ics->num_windows; w += ics->group_len[w]) {
1369  for (int g = 0; g < ics->num_swb; g++) {
1370  int idx = w*16 + g, coded = 0;
1371  for (ch = 0; ch < echans; ch++) {
1372  SingleChannelElement *sce = &ce->ch[ch];
1373  if (sce->zeroes[idx] && sce->band_type[idx] == 0)
1374  continue;
1375  s->stat_ch_bands++;
1376  if (sce->band_type[idx] == NOISE_BT)
1377  s->stat_pns++;
1378  coded = 1;
1379  }
1380  if (etag == TYPE_CPE && coded) {
1381  s->stat_cpe_bands++;
1382  if (ce->ms_mask[idx]) s->stat_ms++;
1383  if (ce->is_mask[idx]) s->stat_is++;
1384  }
1385  }
1386  }
1387  }
1388 
1389  put_bits(&s->pb, 3, TYPE_END);
1390  flush_put_bits(&s->pb);
1391 
1392  s->last_frame_pb_count = put_bits_count(&s->pb);
1393 
1394  /* NMR rate accounting: how many bits the frame really took beyond what the
1395  * trellis counted; feeds the next frame's budget correction */
1396  if (s->nmr) {
1397  int counted = 0;
1398  for (i = 0; i < s->channels; i++)
1399  counted += s->nmr->counted[i];
1400  if (counted > 0) {
1401  float side = (float)s->last_frame_pb_count - counted;
1402  if (s->nmr->side_inited) {
1403  s->nmr->side_ema += 0.125f * (side - s->nmr->side_ema);
1404  } else {
1405  s->nmr->side_ema = side;
1406  s->nmr->side_inited = 1;
1407  }
1408  }
1409  }
1410  avpkt->size = put_bytes_output(&s->pb);
1411 
1412  s->lambda_sum += (s->nmr && s->nmr->lam_rc > 0.0f) ? s->nmr->lam_rc : s->lambda;
1413  s->lambda_count++;
1414 
1415  ff_af_queue_remove(&s->afq, avctx->frame_size, &avpkt->pts,
1416  &avpkt->duration);
1417 
1418  int discard_padding = avctx->frame_size - ff_samples_from_time_base(avctx, avpkt->duration);
1419  if (discard_padding > 0) {
1420  uint8_t *side_data =
1422  if (!side_data)
1423  return AVERROR(ENOMEM);
1424  AV_WL32(side_data + 4, discard_padding);
1425  }
1426 
1427  avpkt->flags |= AV_PKT_FLAG_KEY;
1428 
1429  *got_packet_ptr = 1;
1430  return 0;
1431 }
1432 
1434 {
1435  AACEncContext *s = avctx->priv_data;
1436 
1437  av_log(avctx, AV_LOG_INFO,
1438  "Qavg: %.3f Tr: %.1f%% TNS(L): %.1f%% TNS(S): %.1f%% M/S: %.1f%% I/S: %.1f%% PNS: %.1f%%\n",
1439  s->lambda_count ? s->lambda_sum / s->lambda_count : NAN,
1440  s->stat_chans ? 100.0 * s->stat_short / s->stat_chans : 0.0,
1441  s->stat_chans - s->stat_short ? 100.0 * s->stat_tns_long / (s->stat_chans - s->stat_short) : 0.0,
1442  s->stat_short ? 100.0 * s->stat_tns_short / s->stat_short : 0.0,
1443  s->stat_cpe_bands ? 100.0 * s->stat_ms / s->stat_cpe_bands : 0.0,
1444  s->stat_cpe_bands ? 100.0 * s->stat_is / s->stat_cpe_bands : 0.0,
1445  s->stat_ch_bands ? 100.0 * s->stat_pns / s->stat_ch_bands : 0.0);
1446 
1447  av_tx_uninit(&s->mdct1024);
1448  av_tx_uninit(&s->mdct128);
1449  ff_psy_end(&s->psy);
1450  ff_lpc_end(&s->lpc);
1451  av_freep(&s->buffer.samples);
1452  av_freep(&s->cpe);
1453  av_freep(&s->fdsp);
1454  av_freep(&s->nmr);
1455  ff_af_queue_close(&s->afq);
1456  return 0;
1457 }
1458 
1460 {
1461  int ret = 0;
1462  float scale = 32768.0f;
1463 
1465  if (!s->fdsp)
1466  return AVERROR(ENOMEM);
1467 
1468  if ((ret = av_tx_init(&s->mdct1024, &s->mdct1024_fn, AV_TX_FLOAT_MDCT, 0,
1469  1024, &scale, 0)) < 0)
1470  return ret;
1471  if ((ret = av_tx_init(&s->mdct128, &s->mdct128_fn, AV_TX_FLOAT_MDCT, 0,
1472  128, &scale, 0)) < 0)
1473  return ret;
1474 
1475  return 0;
1476 }
1477 
1479 {
1480  int ch;
1481  if (!FF_ALLOCZ_TYPED_ARRAY(s->buffer.samples, s->channels * 3 * 1024) ||
1482  !FF_ALLOCZ_TYPED_ARRAY(s->cpe, s->chan_map[0]))
1483  return AVERROR(ENOMEM);
1484 
1485  for(ch = 0; ch < s->channels; ch++)
1486  s->planar_samples[ch] = s->buffer.samples + 3 * 1024 * ch;
1487 
1488  if (s->options.coder == AAC_CODER_NMR) {
1489  s->nmr = av_mallocz(sizeof(*s->nmr));
1490  if (!s->nmr)
1491  return AVERROR(ENOMEM);
1492  }
1493 
1494  return 0;
1495 }
1496 
1498 {
1499  AACEncContext *s = avctx->priv_data;
1500  int i, ret = 0;
1501  int chcfg;
1502  const uint8_t *sizes[2];
1503  uint8_t grouping[AAC_MAX_CHANNELS];
1504  int lengths[2];
1505 
1506  /* Constants */
1507  s->last_frame_pb_count = 0;
1508  avctx->frame_size = 1024;
1509  avctx->initial_padding = 1024;
1510  s->lambda = avctx->global_quality > 0 ? avctx->global_quality : 120;
1511 
1512  /* Channel map and unspecified bitrate guessing */
1513  s->channels = avctx->ch_layout.nb_channels;
1514 
1515  s->needs_pce = 1;
1516  for (chcfg = 1; chcfg < FF_ARRAY_ELEMS(aac_normal_chan_layouts); chcfg++) {
1518  s->needs_pce = s->options.pce;
1519  break;
1520  }
1521  }
1522 
1523  if (s->needs_pce) {
1524  char buf[64];
1525  for (i = 0; i < FF_ARRAY_ELEMS(aac_pce_configs); i++)
1527  break;
1528  av_channel_layout_describe(&avctx->ch_layout, buf, sizeof(buf));
1529  if (i == FF_ARRAY_ELEMS(aac_pce_configs)) {
1530  av_log(avctx, AV_LOG_ERROR, "Unsupported channel layout \"%s\"\n", buf);
1531  return AVERROR(EINVAL);
1532  }
1533  av_log(avctx, AV_LOG_INFO, "Using a PCE to encode channel layout \"%s\"\n", buf);
1534  s->pce = aac_pce_configs[i];
1535  s->reorder_map = s->pce.reorder_map;
1536  s->chan_map = s->pce.config_map;
1537  chcfg = 0;
1538  } else {
1539  s->reorder_map = aac_chan_maps[chcfg - 1];
1540  s->chan_map = aac_chan_configs[chcfg - 1];
1541  }
1542 
1543  if (!avctx->bit_rate) {
1544  for (i = 1; i <= s->chan_map[0]; i++) {
1545  avctx->bit_rate += s->chan_map[i] == TYPE_CPE ? 128000 : /* Pair */
1546  s->chan_map[i] == TYPE_LFE ? 16000 : /* LFE */
1547  69000 ; /* SCE */
1548  }
1549  }
1550 
1551  /* Samplerate */
1552  for (int i = 0;; i++) {
1553  av_assert1(i < 13);
1554  if (avctx->sample_rate == ff_mpeg4audio_sample_rates[i]) {
1555  s->samplerate_index = i;
1556  break;
1557  }
1558  }
1559 
1560  /* Bitrate limiting */
1561  WARN_IF(1024.0 * avctx->bit_rate / avctx->sample_rate > 6144 * s->channels,
1562  "Too many bits %f > %d per frame requested, clamping to max\n",
1563  1024.0 * avctx->bit_rate / avctx->sample_rate,
1564  6144 * s->channels);
1565  avctx->bit_rate = (int64_t)FFMIN(6144 * s->channels / 1024.0 * avctx->sample_rate,
1566  avctx->bit_rate);
1567 
1568  /* Profile and option setting */
1569  avctx->profile = avctx->profile == AV_PROFILE_UNKNOWN ? AV_PROFILE_AAC_LOW :
1570  avctx->profile;
1571  for (i = 0; i < FF_ARRAY_ELEMS(aacenc_profiles); i++)
1572  if (avctx->profile == aacenc_profiles[i])
1573  break;
1574  ERROR_IF(i == FF_ARRAY_ELEMS(aacenc_profiles), "Profile not supported!\n");
1575  if (avctx->profile == AV_PROFILE_MPEG2_AAC_LOW) {
1576  avctx->profile = AV_PROFILE_AAC_LOW;
1577  WARN_IF(s->options.pns,
1578  "PNS unavailable in the \"mpeg2_aac_low\" profile, turning off\n");
1579  s->options.pns = 0;
1580  }
1581  s->profile = avctx->profile;
1582 
1583  /* Coder limitations */
1584  s->coder = &ff_aac_coders[s->options.coder];
1585 
1586  /* M/S introduces horrible artifacts with multichannel files, this is temporary */
1587  if (s->channels > 3)
1588  s->options.mid_side = 0;
1589 
1590  /* Coding bandwidth, fixed at init time */
1591  if (avctx->cutoff > 0) {
1592  s->bandwidth = avctx->cutoff;
1593  } else {
1594  int frame_br = (avctx->flags & AV_CODEC_FLAG_QSCALE) ?
1595  (avctx->bit_rate / 2.0f * (s->lambda / 120.f) * 1.5f) :
1596  (avctx->bit_rate / avctx->ch_layout.nb_channels);
1597 
1598  /* For NMR, the rate to bandwidth conversion was tuned to maximize metrics
1599  * over a variable cutoff x bitrate combo */
1600  if (s->options.coder == AAC_CODER_NMR && frame_br >= 32000) {
1601  static const int rates[] = { 32000, 48000, 64000, 96000, 192000 };
1602  static const int bws[] = { 14000, 15000, 16000, 18000, 20000 };
1603  int bw_i = 0;
1604  for (; bw_i < FF_ARRAY_ELEMS(rates) - 2 && frame_br > rates[bw_i + 1]; bw_i++);
1605  s->bandwidth = bws[bw_i] + (int)((int64_t)(bws[bw_i + 1] - bws[bw_i]) *
1606  (frame_br - rates[bw_i]) / (rates[bw_i + 1] - rates[bw_i]));
1607  s->bandwidth = FFMIN3(s->bandwidth, 22000, avctx->sample_rate / 2);
1608  } else {
1609  if (s->options.pns || s->options.intensity_stereo)
1610  frame_br *= 1.15f;
1611  s->bandwidth = FFMAX(3000, AAC_CUTOFF_FROM_BITRATE(frame_br, 1,
1612  avctx->sample_rate));
1613  }
1614 
1615  s->bandwidth = FFMIN(FFMAX(s->bandwidth, 8000), avctx->sample_rate / 2);
1616  }
1617 
1618  // Initialize static tables
1620 
1621  if ((ret = dsp_init(avctx, s)) < 0)
1622  return ret;
1623 
1624  if ((ret = alloc_buffers(avctx, s)) < 0)
1625  return ret;
1626 
1627  if ((ret = put_audio_specific_config(avctx, chcfg)))
1628  return ret;
1629 
1630  sizes[0] = ff_aac_swb_size_1024[s->samplerate_index];
1631  sizes[1] = ff_aac_swb_size_128[s->samplerate_index];
1632  lengths[0] = ff_aac_num_swb_1024[s->samplerate_index];
1633  lengths[1] = ff_aac_num_swb_128[s->samplerate_index];
1634  for (i = 0; i < s->chan_map[0]; i++)
1635  grouping[i] = s->chan_map[i + 1] == TYPE_CPE;
1636  if ((ret = ff_psy_init(&s->psy, avctx, 2, sizes, lengths,
1637  s->chan_map[0], grouping, s->bandwidth)) < 0)
1638  return ret;
1640  s->random_state = 0x1f2e3d4c;
1641 
1642  ff_aacenc_dsp_init(&s->aacdsp);
1643 
1644  ff_af_queue_init(avctx, &s->afq);
1645 
1646  return 0;
1647 }
1648 
1649 #define AACENC_FLAGS AV_OPT_FLAG_ENCODING_PARAM | AV_OPT_FLAG_AUDIO_PARAM
1650 static const AVOption aacenc_options[] = {
1651  {"aac_coder", "Coding algorithm", offsetof(AACEncContext, options.coder), AV_OPT_TYPE_INT, {.i64 = AAC_CODER_NMR}, 0, AAC_CODER_NB-1, AACENC_FLAGS, .unit = "coder"},
1652  {"twoloop", "Two loop searching method", 0, AV_OPT_TYPE_CONST, {.i64 = AAC_CODER_TWOLOOP}, INT_MIN, INT_MAX, AACENC_FLAGS, .unit = "coder"},
1653  {"fast", "Fast search", 0, AV_OPT_TYPE_CONST, {.i64 = AAC_CODER_FAST}, INT_MIN, INT_MAX, AACENC_FLAGS, .unit = "coder"},
1654  {"nmr", "Noise-to-mask ratio scalefactor trellis", 0, AV_OPT_TYPE_CONST, {.i64 = AAC_CODER_NMR}, INT_MIN, INT_MAX, AACENC_FLAGS, .unit = "coder"},
1655  {"aac_ms", "Force M/S stereo coding", offsetof(AACEncContext, options.mid_side), AV_OPT_TYPE_BOOL, {.i64 = -1}, -1, 1, AACENC_FLAGS},
1656  {"aac_is", "Intensity stereo coding", offsetof(AACEncContext, options.intensity_stereo), AV_OPT_TYPE_BOOL, {.i64 = 1}, -1, 1, AACENC_FLAGS},
1657  {"aac_pns", "Perceptual noise substitution", offsetof(AACEncContext, options.pns), AV_OPT_TYPE_BOOL, {.i64 = 1}, -1, 1, AACENC_FLAGS},
1658  {"aac_tns", "Temporal noise shaping", offsetof(AACEncContext, options.tns), AV_OPT_TYPE_BOOL, {.i64 = 1}, -1, 1, AACENC_FLAGS},
1659  {"aac_pce", "Forces the use of PCEs", offsetof(AACEncContext, options.pce), AV_OPT_TYPE_BOOL, {.i64 = 0}, -1, 1, AACENC_FLAGS},
1660  {"aac_nmr_speed", "NMR coder speed level: 0 = slowest/best, higher trades quality for speed", offsetof(AACEncContext, options.nmr_speed), AV_OPT_TYPE_INT, {.i64 = 0}, 0, 4, AACENC_FLAGS},
1662  {NULL}
1663 };
1664 
1665 static const AVClass aacenc_class = {
1666  .class_name = "AAC encoder",
1667  .item_name = av_default_item_name,
1668  .option = aacenc_options,
1669  .version = LIBAVUTIL_VERSION_INT,
1670 };
1671 
1673  { "b", "0" },
1674  { NULL }
1675 };
1676 
1678  .p.name = "aac",
1679  CODEC_LONG_NAME("AAC (Advanced Audio Coding)"),
1680  .p.type = AVMEDIA_TYPE_AUDIO,
1681  .p.id = AV_CODEC_ID_AAC,
1682  .p.capabilities = AV_CODEC_CAP_DR1 | AV_CODEC_CAP_DELAY |
1684  .priv_data_size = sizeof(AACEncContext),
1685  .init = aac_encode_init,
1687  .close = aac_encode_end,
1688  .defaults = aac_encode_defaults,
1690  .caps_internal = FF_CODEC_CAP_INIT_CLEANUP,
1692  .p.priv_class = &aacenc_class,
1693 };
FF_ALLOCZ_TYPED_ARRAY
#define FF_ALLOCZ_TYPED_ARRAY(p, nelem)
Definition: internal.h:72
AVCodecContext::frame_size
int frame_size
Number of samples per channel in an audio frame.
Definition: avcodec.h:1068
AV_SAMPLE_FMT_FLTP
@ AV_SAMPLE_FMT_FLTP
float, planar
Definition: samplefmt.h:66
name
it s the only field you need to keep assuming you have a context There is some magic you don t need to care about around this just let it vf default minimum maximum flags name is the option name
Definition: writing_filters.txt:88
ff_tns_max_bands_128
const uint8_t ff_tns_max_bands_128[]
Definition: aactab.c:1990
AV_CHANNEL_LAYOUT_OCTAGONAL
#define AV_CHANNEL_LAYOUT_OCTAGONAL
Definition: channel_layout.h:422
FF_CODEC_CAP_INIT_CLEANUP
#define FF_CODEC_CAP_INIT_CLEANUP
The codec allows calling the close function for deallocation even if the init function returned a fai...
Definition: codec_internal.h:43
aacenc_class
static const AVClass aacenc_class
Definition: aacenc.c:1665
aac_normal_chan_layouts
static const AVChannelLayout aac_normal_chan_layouts[15]
Definition: aacenctab.h:47
AVERROR
Filter the word “frame” indicates either a video frame or a group of audio as stored in an AVFrame structure Format for each input and each output the list of supported formats For video that means pixel format For audio that means channel sample they are references to shared objects When the negotiation mechanism computes the intersection of the formats supported at each end of a all references to both lists are replaced with a reference to the intersection And when a single format is eventually chosen for a link amongst the remaining all references to the list are updated That means that if a filter requires that its input and output have the same format amongst a supported all it has to do is use a reference to the same list of formats query_formats can leave some formats unset and return AVERROR(EAGAIN) to cause the negotiation mechanism toagain later. That can be used by filters with complex requirements to use the format negotiated on one link to set the formats supported on another. Frame references ownership and permissions
opt.h
SingleChannelElement::can_pns
uint8_t can_pns[128]
band is allowed to PNS (informative)
Definition: aacenc.h:117
LIBAVCODEC_IDENT
#define LIBAVCODEC_IDENT
Definition: version.h:43
AV_WL32
#define AV_WL32(p, v)
Definition: intreadwrite.h:422
put_bitstream_info
static void put_bitstream_info(AACEncContext *s, const char *name)
Write some auxiliary information about the created AAC file.
Definition: aacenc.c:969
ff_aac_kbd_short_128
float ff_aac_kbd_short_128[128]
libm.h
SingleChannelElement::pulse
Pulse pulse
Definition: aacenc.h:112
align_put_bits
static void align_put_bits(PutBitContext *s)
Pad the bitstream with zeros up to the next byte boundary.
Definition: put_bits.h:445
TYPE_FIL
@ TYPE_FIL
Definition: aac.h:50
NMR_IS_FILLMAX
#define NMR_IS_FILLMAX
Definition: aacenc.c:593
ff_af_queue_remove
void ff_af_queue_remove(AudioFrameQueue *afq, int nb_samples, int64_t *pts, int64_t *duration)
Remove frame(s) from the queue.
Definition: audio_frame_queue.c:86
out
static FILE * out
Definition: movenc.c:55
AV_CHANNEL_LAYOUT_STEREO
#define AV_CHANNEL_LAYOUT_STEREO
Definition: channel_layout.h:395
put_bytes_output
static int put_bytes_output(const PutBitContext *s)
Definition: put_bits.h:99
AVCodecContext::sample_rate
int sample_rate
samples per second
Definition: avcodec.h:1040
AV_CHANNEL_LAYOUT_4POINT1
#define AV_CHANNEL_LAYOUT_4POINT1
Definition: channel_layout.h:401
aacenctab.h
AV_CHANNEL_LAYOUT_HEXAGONAL
#define AV_CHANNEL_LAYOUT_HEXAGONAL
Definition: channel_layout.h:411
copy_input_samples
static void copy_input_samples(AACEncContext *s, const AVFrame *frame)
Definition: aacenc.c:990
aac_encode_init
static av_cold int aac_encode_init(AVCodecContext *avctx)
Definition: aacenc.c:1497
nmr_is_image_masked
static int nmr_is_image_masked(AACEncContext *s, ChannelElement *cpe, int w, int g, int start, int len, int gl, float ener0, float ener1, float dot, float minthr0, float minthr1, float *scale_out, float *sr_out, int *p_out)
Definition: aacenc.c:638
aacenc_profiles
static const int aacenc_profiles[]
Definition: aacenctab.h:145
Pulse::num_pulse
int num_pulse
Definition: aac.h:104
AV_CODEC_FLAG_QSCALE
#define AV_CODEC_FLAG_QSCALE
Use fixed qscale.
Definition: avcodec.h:213
av_cold
#define av_cold
Definition: attributes.h:119
int64_t
long long int64_t
Definition: coverity.c:34
output
filter_frame For filters that do not use the this method is called when a frame is pushed to the filter s input It can be called at any time except in a reentrant way If the input frame is enough to produce output
Definition: filter_design.txt:226
SingleChannelElement::zeroes
uint8_t zeroes[128]
band is not coded
Definition: aacenc.h:116
init_put_bits
static void init_put_bits(PutBitContext *s, uint8_t *buffer, int buffer_size)
Initialize the PutBitContext s.
Definition: put_bits.h:62
ff_af_queue_init
av_cold void ff_af_queue_init(AVCodecContext *avctx, AudioFrameQueue *afq)
Initialize AudioFrameQueue.
Definition: audio_frame_queue.c:29
ff_lpc_init
av_cold int ff_lpc_init(LPCContext *s, int blocksize, int max_order, enum FFLPCType lpc_type)
Initialize LPCContext.
Definition: lpc.c:342
AV_CHANNEL_LAYOUT_2_2
#define AV_CHANNEL_LAYOUT_2_2
Definition: channel_layout.h:402
NMR_IS_IMG_GATE
#define NMR_IS_IMG_GATE
Definition: aacenc.c:581
AVFrame
This structure describes decoded (raw) audio or video data.
Definition: frame.h:466
put_bits
static void put_bits(Jpeg2000EncoderContext *s, int val, int n)
put n times val bit
Definition: j2kenc.c:154
WARN_IF
#define WARN_IF(cond,...)
Definition: aacenc_utils.h:250
AVPacket::data
uint8_t * data
Definition: packet.h:603
ff_aac_coders
const AACCoefficientsEncoder ff_aac_coders[AAC_CODER_NB]
Definition: aaccoder.c:817
AVOption
AVOption.
Definition: opt.h:428
encode.h
b
#define b
Definition: input.c:43
R
#define R
Definition: huffyuv.h:44
NMR_MS_MASK
#define NMR_MS_MASK
Definition: aacenc.c:597
encode_band_info
static void encode_band_info(AACEncContext *s, SingleChannelElement *sce)
Encode scalefactor band coding type.
Definition: aacenc.c:831
AV_PROFILE_MPEG2_AAC_LOW
#define AV_PROFILE_MPEG2_AAC_LOW
Definition: defs.h:77
TemporalNoiseShaping::present
int present
Definition: aacdec.h:192
FFCodec
Definition: codec_internal.h:127
version.h
FFPsyWindowInfo::window_shape
int window_shape
window shape (sine/KBD/whatever)
Definition: psymodel.h:79
float.h
AAC_CODER_NB
@ AAC_CODER_NB
Definition: aacenc.h:49
AVPacket::duration
int64_t duration
Duration of this packet in AVStream->time_base units, 0 if unknown.
Definition: packet.h:621
max
#define max(a, b)
Definition: cuda_runtime.h:33
FFMAX
#define FFMAX(a, b)
Definition: macros.h:47
AVChannelLayout::nb_channels
int nb_channels
Number of channels in this layout.
Definition: channel_layout.h:329
ChannelElement::ch
SingleChannelElement ch[2]
Definition: aacdec.h:302
AV_PKT_FLAG_KEY
#define AV_PKT_FLAG_KEY
The packet contains a keyframe.
Definition: packet.h:650
ff_swb_offset_128
const uint16_t *const ff_swb_offset_128[]
Definition: aactab.c:1940
av_tx_init
av_cold int av_tx_init(AVTXContext **ctx, av_tx_fn *tx, enum AVTXType type, int inv, int len, const void *scale, uint64_t flags)
Initialize a transform context with the given configuration (i)MDCTs with an odd length are currently...
Definition: tx.c:903
NMR_IS_FILLGAIN
#define NMR_IS_FILLGAIN
Definition: aacenc.c:592
encode_spectral_coeffs
static void encode_spectral_coeffs(AACEncContext *s, SingleChannelElement *sce)
Encode spectral coefficients processed by psychoacoustic model.
Definition: aacenc.c:900
ff_tns_max_bands_1024
const uint8_t ff_tns_max_bands_1024[]
Definition: aactab.c:1974
AAC_CODER_FAST
@ AAC_CODER_FAST
Definition: aacenc.h:46
IndividualChannelStream::num_swb
int num_swb
number of scalefactor window bands
Definition: aacdec.h:178
AV_CHANNEL_LAYOUT_7POINT1_WIDE
#define AV_CHANNEL_LAYOUT_7POINT1_WIDE
Definition: channel_layout.h:418
WINDOW_FUNC
#define WINDOW_FUNC(type)
Definition: aacenc.c:382
SingleChannelElement::coeffs
float coeffs[1024]
coefficients for IMDCT, maybe processed
Definition: aacenc.h:121
avoid_clipping
static void avoid_clipping(AACEncContext *s, SingleChannelElement *sce)
Downscale spectral coefficients for near-clipping windows to avoid artifacts.
Definition: aacenc.c:928
put_audio_specific_config
static int put_audio_specific_config(AVCodecContext *avctx, int chcfg)
Make AAC audio config object.
Definition: aacenc.c:342
FFCodecDefault
Definition: codec_internal.h:97
FFCodec::p
AVCodec p
The public AVCodec.
Definition: codec_internal.h:131
mpeg4audio.h
b1
static double b1(void *priv, double x, double y)
Definition: vf_xfade.c:2034
AVCodecContext::ch_layout
AVChannelLayout ch_layout
Audio channel layout.
Definition: avcodec.h:1055
SingleChannelElement::ret_buf
float ret_buf[2048]
PCM output buffer.
Definition: aacenc.h:122
apply_mid_side_stereo
static void apply_mid_side_stereo(ChannelElement *cpe)
Definition: aacenc.c:796
AV_CHANNEL_LAYOUT_2POINT1
#define AV_CHANNEL_LAYOUT_2POINT1
Definition: channel_layout.h:396
TYPE_CPE
@ TYPE_CPE
Definition: aac.h:45
ChannelElement::ms_mode
int ms_mode
Signals mid/side stereo flags coding mode.
Definition: aacenc.h:132
AVCodecContext::initial_padding
int initial_padding
Audio only.
Definition: avcodec.h:1114
IndividualChannelStream::window_clipping
uint8_t window_clipping[8]
set if a certain window is near clipping
Definition: aacdec.h:185
AVCodecContext::flags
int flags
AV_CODEC_FLAG_*.
Definition: avcodec.h:500
Pulse::amp
int amp[4]
Definition: aac.h:107
Pulse::pos
int pos[4]
Definition: aac.h:106
AVCodecContext::bit_rate_tolerance
int bit_rate_tolerance
number of bits the bitstream is allowed to diverge from the reference.
Definition: avcodec.h:1227
NMR_IS_LOW_LIMIT
#define NMR_IS_LOW_LIMIT
Definition: aacenc.c:584
put_pce
static void put_pce(PutBitContext *pb, AVCodecContext *avctx)
Definition: aacenc.c:301
ff_psy_end
av_cold void ff_psy_end(FFPsyContext *ctx)
Cleanup model context at the end.
Definition: psymodel.c:77
Pulse::start
int start
Definition: aac.h:105
FF_CODEC_ENCODE_CB
#define FF_CODEC_ENCODE_CB(func)
Definition: codec_internal.h:376
fabsf
static __device__ float fabsf(float a)
Definition: cuda_runtime.h:181
ff_af_queue_add
int ff_af_queue_add(AudioFrameQueue *afq, const AVFrame *f)
Add a frame to the queue.
Definition: audio_frame_queue.c:45
AV_CHANNEL_LAYOUT_6POINT1_FRONT
#define AV_CHANNEL_LAYOUT_6POINT1_FRONT
Definition: channel_layout.h:414
SingleChannelElement::ics
IndividualChannelStream ics
Definition: aacdec.h:218
AV_CHANNEL_LAYOUT_SURROUND
#define AV_CHANNEL_LAYOUT_SURROUND
Definition: channel_layout.h:398
FFPsyWindowInfo
windowing related information
Definition: psymodel.h:77
NMR_PNS_STEREO_DECORR
#define NMR_PNS_STEREO_DECORR
Definition: aacenc.c:604
adjust_frame_information
static void adjust_frame_information(ChannelElement *cpe, int chans)
Produce integer coefficients from scalefactors provided by the model.
Definition: aacenc.c:503
AV_LOG_ERROR
#define AV_LOG_ERROR
Something went wrong and cannot losslessly be recovered.
Definition: log.h:210
FF_ARRAY_ELEMS
#define FF_ARRAY_ELEMS(a)
Definition: sinewin_tablegen.c:29
AV_PROFILE_UNKNOWN
#define AV_PROFILE_UNKNOWN
Definition: defs.h:65
IndividualChannelStream::clip_avoidance_factor
float clip_avoidance_factor
set if any window is near clipping to the necessary atennuation factor to avoid it
Definition: aacenc.h:90
AACPCEInfo::index
uint8_t index[4][8]
front, side, back, lfe
Definition: aacenc.h:196
nmr_apply_is_band
static void nmr_apply_is_band(AACEncContext *s, ChannelElement *cpe, int w, int g, int start, int len, int gl, float scale, float sr_, int p, float ener0, float ener1)
Definition: aacenc.c:670
av_channel_layout_describe
int av_channel_layout_describe(const AVChannelLayout *channel_layout, char *buf, size_t buf_size)
Get a human-readable string describing the channel layout properties.
Definition: channel_layout.c:654
AV_CHANNEL_LAYOUT_4POINT0
#define AV_CHANNEL_LAYOUT_4POINT0
Definition: channel_layout.h:400
float
float
Definition: af_crystalizer.c:122
AVCodecContext::extradata_size
int extradata_size
Definition: avcodec.h:527
NOISE_BT
@ NOISE_BT
Spectral data are scaled white noise not coded in the bitstream.
Definition: aac.h:75
AV_TX_FLOAT_MDCT
@ AV_TX_FLOAT_MDCT
Standard MDCT with a sample data type of float, double or int32_t, respectively.
Definition: tx.h:68
AV_CHANNEL_LAYOUT_7POINT1
#define AV_CHANNEL_LAYOUT_7POINT1
Definition: channel_layout.h:417
AVCodecContext::global_quality
int global_quality
Global quality for codecs which cannot change it per frame.
Definition: avcodec.h:1235
IndividualChannelStream::swb_sizes
const uint8_t * swb_sizes
table of scalefactor band sizes for a particular window
Definition: aacenc.h:85
g
const char * g
Definition: vf_curves.c:128
AVMEDIA_TYPE_AUDIO
@ AVMEDIA_TYPE_AUDIO
Definition: avutil.h:201
EIGHT_SHORT_SEQUENCE
@ EIGHT_SHORT_SEQUENCE
Definition: aac.h:66
info
MIPS optimizations info
Definition: mips.txt:2
AV_CHANNEL_LAYOUT_5POINT0_BACK
#define AV_CHANNEL_LAYOUT_5POINT0_BACK
Definition: channel_layout.h:406
INTENSITY_BT2
@ INTENSITY_BT2
Scalefactor data are intensity stereo positions (out of phase).
Definition: aac.h:76
av_assert0
#define av_assert0(cond)
assert() equivalent, that is always enabled.
Definition: avassert.h:42
alloc_buffers
static av_cold int alloc_buffers(AVCodecContext *avctx, AACEncContext *s)
Definition: aacenc.c:1478
channels
channels
Definition: aptx.h:31
channel_map
static const uint8_t channel_map[8][8]
Definition: atrac3plusdec.c:52
ff_put_string
void ff_put_string(PutBitContext *pb, const char *string, int terminate_string)
Put the string string in the bitstream.
Definition: bitstream.c:39
ff_samples_from_time_base
static av_always_inline int64_t ff_samples_from_time_base(const AVCodecContext *avctx, int64_t duration)
Rescale from time base to AVCodecContext.sample_rate.
Definition: encode.h:105
IndividualChannelStream
Individual Channel Stream.
Definition: aacdec.h:169
av_mallocz
#define av_mallocz(s)
Definition: tableprint_vlc.h:31
SCALE_DIFF_ZERO
#define SCALE_DIFF_ZERO
codebook index corresponding to zero scalefactor indices difference
Definition: aac.h:95
NAN
#define NAN
Definition: mathematics.h:115
NOISE_PRE
#define NOISE_PRE
preamble for NOISE_BT, put in bitstream with the first noise band
Definition: aac.h:99
PutBitContext
Definition: put_bits.h:50
CODEC_LONG_NAME
#define CODEC_LONG_NAME(str)
Definition: codec_internal.h:349
if
if(ret)
Definition: filter_design.txt:179
ff_af_queue_close
av_cold void ff_af_queue_close(AudioFrameQueue *afq)
Close AudioFrameQueue.
Definition: audio_frame_queue.c:37
AV_CHANNEL_LAYOUT_7POINT1_WIDE_BACK
#define AV_CHANNEL_LAYOUT_7POINT1_WIDE_BACK
Definition: channel_layout.h:419
INTENSITY_BT
@ INTENSITY_BT
Scalefactor data are intensity stereo positions (in phase).
Definition: aac.h:77
FFPsyWindowInfo::window_type
int window_type[3]
window type (short/long/transitional, etc.) - current, previous and next
Definition: psymodel.h:78
AAC_MAX_CHANNELS
#define AAC_MAX_CHANNELS
Definition: aacenctab.h:41
LIBAVUTIL_VERSION_INT
#define LIBAVUTIL_VERSION_INT
Definition: version.h:85
AVClass
Describe the class of an AVClass context structure.
Definition: log.h:76
fabs
static __device__ float fabs(float a)
Definition: cuda_runtime.h:182
ChannelElement::is_mask
uint8_t is_mask[128]
Set if intensity stereo is used.
Definition: aacenc.h:135
NULL
#define NULL
Definition: coverity.c:32
AACPCEInfo::pairing
uint8_t pairing[3][8]
front, side, back
Definition: aacenc.h:195
sizes
static const int sizes[][2]
Definition: img2dec.c:62
NMR_IS_MAXBPS
#define NMR_IS_MAXBPS
Definition: aacenc.c:587
encode_pulses
static void encode_pulses(AACEncContext *s, Pulse *pulse)
Encode pulse data.
Definition: aacenc.c:881
SingleChannelElement::is_ener
float is_ener[128]
Intensity stereo pos.
Definition: aacenc.h:118
IndividualChannelStream::use_kb_window
uint8_t use_kb_window[2]
If set, use Kaiser-Bessel window, otherwise use a sine window.
Definition: aacdec.h:172
ff_aac_num_swb_128
const uint8_t ff_aac_num_swb_128[]
Definition: aactab.c:169
AVCodecContext::bit_rate
int64_t bit_rate
the average bitrate
Definition: avcodec.h:493
av_default_item_name
const char * av_default_item_name(void *ptr)
Return the context name.
Definition: log.c:242
profiles.h
ff_lpc_end
av_cold void ff_lpc_end(LPCContext *s)
Uninitialize LPCContext.
Definition: lpc.c:367
ChannelElement::ms_mask
uint8_t ms_mask[128]
Set if mid/side stereo is used for each scalefactor window band.
Definition: aacdec.h:300
options
Definition: swscale.c:50
FFPsyBand
single band psychoacoustic information
Definition: psymodel.h:50
aac.h
aactab.h
nmr_decide_stereo
static void nmr_decide_stereo(AACEncContext *s, ChannelElement *cpe)
Definition: aacenc.c:700
sqrtf
static __device__ float sqrtf(float a)
Definition: cuda_runtime.h:184
FFPsyWindowInfo::grouping
int grouping[8]
window grouping (for e.g. AAC)
Definition: psymodel.h:81
av_clipf
av_clipf
Definition: af_crystalizer.c:122
TNS_MAX_ORDER
#define TNS_MAX_ORDER
Definition: aac.h:36
c
Undefined Behavior In the C some operations are like signed integer dereferencing freed accessing outside allocated Undefined Behavior must not occur in a C it is not safe even if the output of undefined operations is unused The unsafety may seem nit picking but Optimizing compilers have in fact optimized code on the assumption that no undefined Behavior occurs Optimizing code based on wrong assumptions can and has in some cases lead to effects beyond the output of computations The signed integer overflow problem in speed critical code Code which is highly optimized and works with signed integers sometimes has the problem that often the output of the computation does not c
Definition: undefined.txt:32
SingleChannelElement::sf_idx
int sf_idx[128]
scalefactor indices
Definition: aacenc.h:115
float_dsp.h
AV_CODEC_ID_AAC
@ AV_CODEC_ID_AAC
Definition: codec_id.h:454
aac_encode_frame
static int aac_encode_frame(AVCodecContext *avctx, AVPacket *avpkt, const AVFrame *frame, int *got_packet_ptr)
Definition: aacenc.c:1012
ff_aac_scalefactor_bits
const uint8_t ff_aac_scalefactor_bits[121]
Definition: aactab.c:200
NMR_MS_EQUIV
#define NMR_MS_EQUIV
Definition: aacenc.c:596
AACPCEInfo
Definition: aacenc.h:192
FFPsyWindowInfo::clipping
float clipping[8]
maximum absolute normalized intensity in the given window for clip avoidance
Definition: psymodel.h:82
IndividualChannelStream::window_sequence
enum WindowSequence window_sequence[2]
Definition: aacdec.h:171
f
f
Definition: af_crystalizer.c:122
init
int(* init)(AVBSFContext *ctx)
Definition: dts2pts.c:579
AV_CODEC_CAP_DR1
#define AV_CODEC_CAP_DR1
Codec uses get_buffer() or get_encode_buffer() for allocating buffers and supports custom allocators.
Definition: codec.h:49
ff_swb_offset_1024
const uint16_t *const ff_swb_offset_1024[]
Definition: aactab.c:1900
AVPacket::size
int size
Definition: packet.h:604
codec_internal.h
ONLY_LONG_SEQUENCE
@ ONLY_LONG_SEQUENCE
Definition: aac.h:64
TYPE_END
@ TYPE_END
Definition: aac.h:51
ff_aac_float_common_init
void ff_aac_float_common_init(void)
i
#define i(width, name, range_min, range_max)
Definition: cbs_h264.c:63
encode_scale_factors
static void encode_scale_factors(AVCodecContext *avctx, AACEncContext *s, SingleChannelElement *sce)
Encode scalefactors.
Definition: aacenc.c:845
for
for(k=2;k<=8;++k)
Definition: h264pred_template.c:424
bps
unsigned bps
Definition: movenc.c:2074
apply_window_and_mdct
static void apply_window_and_mdct(AACEncContext *s, SingleChannelElement *sce, float *audio)
Definition: aacenc.c:447
AVFloatDSPContext
Definition: float_dsp.h:24
AAC_CODER_TWOLOOP
@ AAC_CODER_TWOLOOP
Definition: aacenc.h:45
aac_chan_configs
static const uint8_t aac_chan_configs[14][6]
default channel configurations
Definition: aacenctab.h:66
AV_CHANNEL_LAYOUT_6POINT0
#define AV_CHANNEL_LAYOUT_6POINT0
Definition: channel_layout.h:408
diff
static av_always_inline int diff(const struct color_info *a, const struct color_info *b, const int trans_thresh)
Definition: vf_paletteuse.c:166
CLIP_AVOIDANCE_FACTOR
#define CLIP_AVOIDANCE_FACTOR
Definition: aacenc.h:42
ChannelElement::common_window
int common_window
Set if channels share a common 'IndividualChannelStream' in bitstream.
Definition: aacenc.h:131
sinewin.h
apply_intensity_stereo
static void apply_intensity_stereo(ChannelElement *cpe)
Definition: aacenc.c:551
AVPacket::flags
int flags
A combination of AV_PKT_FLAG values.
Definition: packet.h:609
CODEC_SAMPLEFMTS
#define CODEC_SAMPLEFMTS(...)
Definition: codec_internal.h:395
nmr_apply_ms_band
static void nmr_apply_ms_band(AACEncContext *s, ChannelElement *cpe, int w, int g, int start, int len, int gl)
Definition: aacenc.c:610
SingleChannelElement::band_type
enum BandType band_type[128]
band types
Definition: aacdec.h:221
av_tx_uninit
av_cold void av_tx_uninit(AVTXContext **ctx)
Frees a context and sets *ctx to NULL, does nothing when *ctx == NULL.
Definition: tx.c:295
av_channel_layout_compare
int av_channel_layout_compare(const AVChannelLayout *chl, const AVChannelLayout *chl1)
Check whether two channel layouts are semantically the same, i.e.
Definition: channel_layout.c:811
AV_LOG_INFO
#define AV_LOG_INFO
Standard information.
Definition: log.h:221
AAC_CUTOFF_FROM_BITRATE
#define AAC_CUTOFF_FROM_BITRATE(bit_rate, channels, sample_rate)
Definition: psymodel.h:35
AV_CHANNEL_LAYOUT_6POINT1_BACK
#define AV_CHANNEL_LAYOUT_6POINT1_BACK
Definition: channel_layout.h:413
aac_pce_configs
static const AACPCEInfo aac_pce_configs[]
List of PCE (Program Configuration Element) for the channel layouts listed in channel_layout....
Definition: aacenc.c:90
SingleChannelElement
Single Channel Element - used for both SCE and LFE elements.
Definition: aacdec.h:217
AVPacket::pts
int64_t pts
Presentation timestamp in AVStream->time_base units; the time at which the decompressed packet will b...
Definition: packet.h:596
put_bits_count
static int put_bits_count(PutBitContext *s)
Definition: put_bits.h:90
IndividualChannelStream::num_windows
int num_windows
Definition: aacdec.h:179
AVCodecContext::extradata
uint8_t * extradata
Out-of-band global headers that may be used by some codecs.
Definition: avcodec.h:526
FFMIN3
#define FFMIN3(a, b, c)
Definition: macros.h:50
aacenc_options
static const AVOption aacenc_options[]
Definition: aacenc.c:1650
AV_CHANNEL_LAYOUT_QUAD
#define AV_CHANNEL_LAYOUT_QUAD
Definition: channel_layout.h:403
SingleChannelElement::pcoeffs
float pcoeffs[1024]
coefficients for IMDCT, pristine
Definition: aacenc.h:120
LONG_STOP_SEQUENCE
@ LONG_STOP_SEQUENCE
Definition: aac.h:67
ChannelElement
channel element - generic struct for SCE/CPE/CCE/LFE
Definition: aacdec.h:296
IndividualChannelStream::swb_offset
const uint16_t * swb_offset
table of offsets to the lowest spectral coefficient of a scalefactor band, sfb, for a particular wind...
Definition: aacdec.h:177
ff_psy_init
av_cold int ff_psy_init(FFPsyContext *ctx, AVCodecContext *avctx, int num_lens, const uint8_t **bands, const int *num_bands, int num_groups, const uint8_t *group_map, int cutoff)
Initialize psychoacoustic model.
Definition: psymodel.c:28
AVCodecContext::cutoff
int cutoff
Audio cutoff bandwidth (0 means "automatic")
Definition: avcodec.h:1082
AV_CHANNEL_LAYOUT_7POINT0_FRONT
#define AV_CHANNEL_LAYOUT_7POINT0_FRONT
Definition: channel_layout.h:416
apply_window
static void(*const apply_window[4])(AVFloatDSPContext *fdsp, SingleChannelElement *sce, const float *audio)
Definition: aacenc.c:438
av_assert1
#define av_assert1(cond)
assert() equivalent, that does not lie in speed critical code.
Definition: avassert.h:58
s
uint8_t s
Definition: llvidencdsp.c:39
NOISE_PRE_BITS
#define NOISE_PRE_BITS
length of preamble
Definition: aac.h:100
AV_CHANNEL_LAYOUT_3POINT1
#define AV_CHANNEL_LAYOUT_3POINT1
Definition: channel_layout.h:399
FFMIN
#define FFMIN(a, b)
Definition: macros.h:49
TYPE_LFE
@ TYPE_LFE
Definition: aac.h:47
ff_aac_kbd_long_1024
float ff_aac_kbd_long_1024[1024]
AACPCEInfo::num_ele
uint8_t num_ele[4]
front, side, back, lfe
Definition: aacenc.h:194
AVCodec::name
const char * name
Name of the codec implementation.
Definition: codec.h:176
TYPE_SCE
@ TYPE_SCE
Definition: aac.h:44
AACENC_FLAGS
#define AACENC_FLAGS
Definition: aacenc.c:1649
len
int len
Definition: vorbis_enc_data.h:426
IndividualChannelStream::tns_max_bands
int tns_max_bands
Definition: aacdec.h:180
AAC_CODER_NMR
@ AAC_CODER_NMR
Definition: aacenc.h:47
avcodec.h
AVCodecContext::frame_num
int64_t frame_num
Frame counter, set by libavcodec.
Definition: avcodec.h:1883
aac_encode_defaults
static const FFCodecDefault aac_encode_defaults[]
Definition: aacenc.c:1672
tag
uint32_t tag
Definition: movenc.c:2073
ret
ret
Definition: filter_design.txt:187
ff_aac_num_swb_1024
const uint8_t ff_aac_num_swb_1024[]
Definition: aactab.c:149
AVClass::class_name
const char * class_name
The name of the class; usually it is the same name as the context structure type to which the AVClass...
Definition: log.h:81
frame
these buffered frames must be flushed immediately if a new input produces new the filter must not call request_frame to get more It must just process the frame or queue it The task of requesting more frames is left to the filter s request_frame method or the application If a filter has several the filter must be ready for frames arriving randomly on any input any filter with several inputs will most likely require some kind of queuing mechanism It is perfectly acceptable to have a limited queue and to drop frames when the inputs are too unbalanced request_frame For filters that do not use the this method is called when a frame is wanted on an output For a it should directly call filter_frame on the corresponding output For a if there are queued frames already one of these frames should be pushed If the filter should request a frame on one of its repeatedly until at least one frame has been pushed Return or at least make progress towards producing a frame
Definition: filter_design.txt:265
ff_aac_encoder
const FFCodec ff_aac_encoder
Definition: aacenc.c:1677
encode_ms_info
static void encode_ms_info(PutBitContext *pb, ChannelElement *cpe)
Encode MS data.
Definition: aacenc.c:489
AV_CHANNEL_LAYOUT_7POINT0
#define AV_CHANNEL_LAYOUT_7POINT0
Definition: channel_layout.h:415
RESERVED_BT
@ RESERVED_BT
Band types following are encoded differently from others.
Definition: aac.h:74
LONG_START_SEQUENCE
@ LONG_START_SEQUENCE
Definition: aac.h:65
SingleChannelElement::tns
TemporalNoiseShaping tns
Definition: aacdec.h:220
AACEncContext
AAC encoder context.
Definition: aacenc.h:204
AV_PROFILE_AAC_LOW
#define AV_PROFILE_AAC_LOW
Definition: defs.h:69
AV_CHANNEL_LAYOUT_2_1
#define AV_CHANNEL_LAYOUT_2_1
Definition: channel_layout.h:397
AVCodecContext
main external API structure.
Definition: avcodec.h:443
channel_layout.h
CODEC_SAMPLERATES_ARRAY
#define CODEC_SAMPLERATES_ARRAY(array)
Definition: codec_internal.h:393
encode_individual_channel
static int encode_individual_channel(AVCodecContext *avctx, AACEncContext *s, SingleChannelElement *sce, int common_window)
Encode one channel of audio data.
Definition: aacenc.c:948
av_packet_new_side_data
uint8_t * av_packet_new_side_data(AVPacket *pkt, enum AVPacketSideDataType type, size_t size)
Allocate new information of a packet.
Definition: packet.c:231
NOISE_OFFSET
#define NOISE_OFFSET
subtracted from global gain, used as offset for the preamble
Definition: aac.h:101
ERROR_IF
#define ERROR_IF(cond,...)
Definition: aacenc_utils.h:244
rates
static const int rates[]
Definition: swresample.c:101
ff_aac_swb_size_1024
const uint8_t *const ff_aac_swb_size_1024[]
Definition: aacenctab.c:97
AV_OPT_TYPE_INT
@ AV_OPT_TYPE_INT
Underlying C type is int.
Definition: opt.h:258
TemporalNoiseShaping
Temporal Noise Shaping.
Definition: aacdec.h:191
AVCodecContext::profile
int profile
profile
Definition: avcodec.h:1636
AOT_SBR
@ AOT_SBR
Y Spectral Band Replication.
Definition: mpeg4audio.h:78
AV_PKT_DATA_SKIP_SAMPLES
@ AV_PKT_DATA_SKIP_SAMPLES
Recommends skipping the specified number of samples.
Definition: packet.h:153
L
#define L(x)
Definition: vpx_arith.h:36
AV_CHANNEL_LAYOUT_6POINT0_FRONT
#define AV_CHANNEL_LAYOUT_6POINT0_FRONT
Definition: channel_layout.h:409
AV_CODEC_CAP_DELAY
#define AV_CODEC_CAP_DELAY
Encoder or decoder requires flushing with NULL input at the end in order to give the complete and cor...
Definition: codec.h:73
samples
Filter the word “frame” indicates either a video frame or a group of audio samples
Definition: filter_design.txt:8
ChannelElement::is_mode
uint8_t is_mode
Set if any bands have been encoded using intensity stereo.
Definition: aacenc.h:133
Windows::Graphics::DirectX::Direct3D11::p
IDirect3DDxgiInterfaceAccess _COM_Outptr_ void ** p
Definition: vsrc_gfxcapture_winrt.hpp:53
ff_aacenc_dsp_init
void ff_aacenc_dsp_init(AACEncDSPContext *s)
Definition: aacencdsp.c:75
put_ics_info
static void put_ics_info(AACEncContext *s, IndividualChannelStream *info)
Encode ics_info element.
Definition: aacenc.c:468
ff_mpeg4audio_sample_rates
const int ff_mpeg4audio_sample_rates[16]
Definition: mpeg4audio_sample_rates.h:30
ff_aac_swb_size_128
const uint8_t *const ff_aac_swb_size_128[]
Definition: aacenctab.c:89
mem.h
AV_CODEC_FLAG_BITEXACT
#define AV_CODEC_FLAG_BITEXACT
Use only bitexact stuff (except (I)DCT).
Definition: avcodec.h:322
aac_encode_end
static av_cold int aac_encode_end(AVCodecContext *avctx)
Definition: aacenc.c:1433
flush_put_bits
static void flush_put_bits(PutBitContext *s)
Pad the end of the output stream with zeros.
Definition: put_bits.h:153
w
uint8_t w
Definition: llvidencdsp.c:39
AV_CHANNEL_LAYOUT_MONO
#define AV_CHANNEL_LAYOUT_MONO
Definition: channel_layout.h:394
scale
static void scale(int *out, const int *in, const int w, const int h, const int shift)
Definition: intra.c:278
FF_AAC_PROFILE_OPTS
#define FF_AAC_PROFILE_OPTS
Definition: profiles.h:29
AVPacket
This structure stores compressed data.
Definition: packet.h:580
AVCodecContext::priv_data
void * priv_data
Definition: avcodec.h:470
AV_OPT_TYPE_BOOL
@ AV_OPT_TYPE_BOOL
Underlying C type is int.
Definition: opt.h:326
av_freep
#define av_freep(p)
Definition: tableprint_vlc.h:35
avpriv_float_dsp_alloc
av_cold AVFloatDSPContext * avpriv_float_dsp_alloc(int bit_exact)
Allocate a float DSP context.
Definition: float_dsp.c:135
AV_CHANNEL_LAYOUT_5POINT1_BACK
#define AV_CHANNEL_LAYOUT_5POINT1_BACK
Definition: channel_layout.h:407
IndividualChannelStream::max_sfb
uint8_t max_sfb
number of scalefactor bands per group
Definition: aacdec.h:170
Pulse
Definition: aac.h:103
av_log
#define av_log(a,...)
Definition: tableprint_vlc.h:27
AV_CHANNEL_LAYOUT_6POINT1
#define AV_CHANNEL_LAYOUT_6POINT1
Definition: channel_layout.h:412
b0
static double b0(void *priv, double x, double y)
Definition: vf_xfade.c:2033
dsp_init
static av_cold int dsp_init(AVCodecContext *avctx, AACEncContext *s)
Definition: aacenc.c:1459
AV_CHANNEL_LAYOUT_5POINT0
#define AV_CHANNEL_LAYOUT_5POINT0
Definition: channel_layout.h:404
aacenc_utils.h
aac_chan_maps
static const uint8_t aac_chan_maps[14][AAC_MAX_CHANNELS]
Table to remap channels from libavcodec's default order to AAC order.
Definition: aacenctab.h:86
AV_CODEC_CAP_SMALL_LAST_FRAME
#define AV_CODEC_CAP_SMALL_LAST_FRAME
Codec can be fed a final frame with a smaller size.
Definition: codec.h:78
AV_CHANNEL_LAYOUT_5POINT1
#define AV_CHANNEL_LAYOUT_5POINT1
Definition: channel_layout.h:405
put_bits.h
IndividualChannelStream::group_len
uint8_t group_len[8]
Definition: aacdec.h:175
psymodel.h
AV_OPT_TYPE_CONST
@ AV_OPT_TYPE_CONST
Special option type for declaring named constants.
Definition: opt.h:298
ff_alloc_packet
int ff_alloc_packet(AVCodecContext *avctx, AVPacket *avpkt, int64_t size)
Check AVPacket size and allocate data.
Definition: encode.c:61
FF_LPC_TYPE_LEVINSON
@ FF_LPC_TYPE_LEVINSON
Levinson-Durbin recursion.
Definition: lpc.h:46
FFPsyWindowInfo::num_windows
int num_windows
number of windows in a frame
Definition: psymodel.h:80
ff_aac_scalefactor_code
const uint32_t ff_aac_scalefactor_code[121]
Definition: aactab.c:181
ff_quantize_band_cost_cache_init
void ff_quantize_band_cost_cache_init(struct AACEncContext *s)
Definition: aacenc.c:373
AACPCEInfo::layout
AVChannelLayout layout
Definition: aacenc.h:193
aacenc.h